#Test examples in ...
import sys
import cStringIO
import unittest
from util import run_stron

ENCODING = 'utf-8'


STRON1 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Special XHTML conventions</title>
  <ns uri="http://www.w3.org/1999/xhtml" prefix="html"/>
  <pattern name="Document head">
    <rule context="html:head">
      <assert test="html:title">Page does not have a title.</assert>
      <assert test="html:link/@rel = 'stylesheet'
                    and html:link/@type = 'text/css'
                    and html:link/@href = 'std.css'">
        Page does not use the standard stylesheet.
      </assert>
      <report test="html:style">
        Page uses in-line style rather than linking to the
        standard stylesheet.
      </report>
    </rule>
  </pattern>
  <pattern name="Document body">
    <rule context="html:body">
      <assert test="@class = 'std-body'">
        Page does not use the standard body class.
      </assert>
      <assert test="html:*[1]/self::html:div[@class = 'std-top']">
        Page does not start with the required page top component
      </assert>
    </rule>
  </pattern>
</schema>
'''

DTDECL = u'''\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
'''

INSTANCE1 = u'''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link rel="stylesheet" type="text/css" href="std.css"/>
    <title>Document head</title>
  </head>
  <body class="std-body">
    <div class="std-top">Top component</div>
  </body>
</html>'''.encode(ENCODING)


INSTANCE2 = u'''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>Document head</title>
    <style>
      body.std-body { background white; }
    </style>
  </head>
  <body class="std-body">
    <h1>Title</h1>
    <div class="std-top">Top component</div>
  </body>
</html>'''.encode(ENCODING)


INSTANCE3 = u'''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link rel="stylesheet" type="text/css" href="std.css"/>
  </head>
  <body>
    <div class="std-top">Top component</div>
  </body>
</html>'''.encode(ENCODING)

EXAMPLES = []

EG_3_1 = (
'eg3_1',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Document root">
    <rule context="/">
      <assert test="doc">Root element should be "doc".</assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'<doc/>'.encode(ENCODING), '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\n'),
),
(
(u'<bogus/>'.encode(ENCODING), '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nAssertion failure:\nRoot element should be "doc".\n'),
)
)

EXAMPLES += [EG_3_1]

EG_3_2 = (
'eg3_2',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Major elements">
    <rule context="doc">
      <assert test="prologue">
        The "doc" element must have a "prologue" child.
      </assert>
      <assert test="section">
        The "doc" element must have at least one "section" child.
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\n'),
(u'''\
<doc>
  <prologue/>
  <section/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\n')
),
(
(u'''\
<doc>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\nThe "doc" element must have a "prologue" child.\n'),
(u'''\
<doc>
  <prologue/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\nThe "doc" element must have at least one "section" child.\n'),
(u'''\
<doc/>
'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\nThe "doc" element must have a "prologue" child.\nAssertion failure:\nThe "doc" element must have at least one "section" child.\n')
)
)

EXAMPLES += [EG_3_2]

EG_3_3 = (
'eg3_3',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Extaneous docs">
    <rule context="doc">
      <assert test="not(ancestor::*)">
        The "doc" element is only allowed at the document root.
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'<doc><ok/></doc>'.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Extaneous docs\n'),
),
(
(u'<doc><doc/></doc>'.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Extaneous docs\nAssertion failure:\nThe "doc" element is only allowed at the document root.\n'),
)
)

EXAMPLES += [EG_3_3]

EG_3_4 = (
'eg3_4',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Extraneous docs">
    <rule context="prologue">
      <assert test="not(preceding-sibling::section)">
        No "section" may occur before the "prologue".
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue/>
  <section/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Extraneous docs\n'),
),
(
(u'''\
<doc>
  <section/>
  <prologue/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Extraneous docs\nAssertion failure:\nNo "section" may occur before the "prologue".\n'),
)
)

EXAMPLES += [EG_3_4]

EG_3_5 = (
'eg3_5',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Title with subtitle">
    <rule context="title">
      <assert test="following-sibling::*[1]/self::subtitle">
        A "title" must be immediately followed by a "subtitle".
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Title with subtitle\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Title with subtitle\nAssertion failure:\nA "title" must be immediately followed by a "subtitle".\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <midtitle>Seriously</midtitle>
    <subtitle>From fantasy to reality</subtitle>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Title with subtitle\nAssertion failure:\nA "title" must be immediately followed by a "subtitle".\n'),
)
)

EXAMPLES += [EG_3_5]

EG_3_6 = (
'eg3_6',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Minimum keywords">
    <rule context="prologue">
      <assert test="count(keyword) > 2">
        At least three keywords are required.
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <keyword>tachyon</keyword>
    <keyword>higgs</keyword>
    <keyword>propulsion</keyword>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Minimum keywords\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <keyword>tachyon</keyword>
    <keyword>higgs</keyword>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Minimum keywords\nAssertion failure:\nAt least three keywords are required.\n'),
)
)

EXAMPLES += [EG_3_6]

EG_3_7 = (
'eg3_7',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Author attributes">
    <rule context="author">
      <assert test="@e-mail">
        author must have e-mail attribute.
      </assert>
      <assert test="@member = 'yes' or @member = 'no'">
        author must have member attribute with 'yes' or 'no' value.
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Author attributes\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes'>Chikezie Emereuwa</author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Author attributes\nAssertion failure:\nauthor must have e-mail attribute.\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Author attributes\nAssertion failure:\nauthor must have member attribute with \'yes\' or \'no\' value.\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='maybe' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Author attributes\nAssertion failure:\nauthor must have member attribute with \'yes\' or \'no\' value.\n'),
)
)

EXAMPLES += [EG_3_7]

EG_3_8 = (
'eg3_8',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Useful title">
    <rule context="title">
      <assert test="text()">
        title may not be empty
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Useful title\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title></title>
    <subtitle>From fantasy to reality</subtitle>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Useful title\nAssertion failure:\ntitle may not be empty\n'),
)
)

EXAMPLES += [EG_3_8]

EG_3_9 = (
'eg3_9',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Author elements">
    <rule context="author">
      <assert test="count(name|bio|affiliation) = count(*)">
        Only "name", "bio" and "affiliation" elements are allowed as children of "author"
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">
      <name>Chikezie Emereuwa</name>
      <bio>Chikezie Emereuwa is a quantum engineer and NASA researcher</bio>
      <affiliation>NASA</affiliation>
    </author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Author elements\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">
      <name>Chikezie Emereuwa</name>
      <bio>Chikezie Emereuwa is a quantum engineer and NASA researcher</bio>
      <affiliation>NASA</affiliation>
      <age>39</age>
    </author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Author elements\nAssertion failure:\nOnly "name", "bio" and "affiliation" elements are allowed as children of "author"\n'),
)
)

EXAMPLES += [EG_3_9]

EG_4_1 = (
'eg4_1',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Military authors">
    <rule context="author">
      <!-- This test should really be refined so it
           checks that '.mil' is in the last position -->
      <report test="contains(@e-mail, '.mil')">
        Author appears to be military personnel
      </report>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
    <author member='yes' e-mail="okey.agu@navy.mil">Okechukwu Agu</author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Military authors\nReport:\nAuthor appears to be military personnel\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Military authors\n'),
),
()
)

EXAMPLES += [EG_4_1]

EG_4_2 = (
'eg4_2',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Report links">
    <rule context="*">
      <report test="@link">
        <name/> element has a link.
      </report>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue/>
  <section>
    Placeholder for the <emphasis link='http://nasa.gov/ftl/paper.xml'>actual content</emphasis>.
  </section>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Report links\nReport:\nemphasis element has a link.\n'),
),
()
)

EXAMPLES += [EG_4_2]

EG_4_3 = (
'eg4_3',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Title with subtitle">
    <rule context="title">
      <assert test="following-sibling::*[1]/self::subtitle">
        A <name/> must be immediately followed by a "subtitle",
        not <name path="following-sibling::*[1]"/>.
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Title with subtitle\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Title with subtitle\nAssertion failure:\nA title must be immediately followed by a "subtitle",\nnot .\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <midtitle>Seriously</midtitle>
    <subtitle>From fantasy to reality</subtitle>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Title with subtitle\nAssertion failure:\nA title must be immediately followed by a "subtitle",\nnot midtitle.\n'),
)
)

EXAMPLES += [EG_4_3]

EG_4_4 = (
'eg4_4',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Report links">
    <rule context="*">
      <report test="@link">
        <name/> element has a link to <value-of select="@link"/>.
      </report>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue/>
  <section>
    Placeholder for the <emphasis link='http://nasa.gov/ftl/paper.xml'>actual content</emphasis>.
  </section>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Report links\nReport:\nemphasis element has a link to http://nasa.gov/ftl/paper.xml.\n'),
),
()
)

EXAMPLES += [EG_4_4]

EG_4_5 = (
'eg4_5',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Major elements">
    <rule context="doc">
      <assert test="prologue" diagnostics="doc-struct">
        <name/> element must have a prologue.
      </assert>
      <assert test="section" diagnostics="doc-struct sect">
        <name/> element must have at least one section.
      </assert>
    </rule>
  </pattern>
  <diagnostics>
    <diagnostic id="doc-struct">
       A document (the <name/> element) must have a prologue and one or
       more sections.  Please correct your submission by adding the
       required elements, then re-submit.  For your records, the
       submission ID is <value-of select="@id"/>.
    </diagnostic>
    <diagnostic id="sect">
       Sections are sometimes omitted because authors think they can
       just place the document contents directly after the prologue.
       You may be able to correct this error by just wrapping your
       existing content in a section element.
    </diagnostic>
  </diagnostics>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc id='ftl2004'>
  <prologue/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\n'),
(u'''\
<doc id='ftl2004'>
  <prologue/>
  <section/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\n'),
),
(
(u'''\
<doc id='ftl2004'>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\ndoc element must have a prologue.\nDiagnostic message:\nA document (the doc element) must have a prologue and one or\nmore sections. Please correct your submission by adding the\nrequired elements, then re-submit. For your records, the\nsubmission ID is ftl2004.\n'),
(u'''\
<doc id='ftl2004'>
  <prologue/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\ndoc element must have at least one section.\nDiagnostic message:\nA document (the doc element) must have a prologue and one or\nmore sections. Please correct your submission by adding the\nrequired elements, then re-submit. For your records, the\nsubmission ID is ftl2004.\nDiagnostic message:\nSections are sometimes omitted because authors think they can\njust place the document contents directly after the prologue.\nYou may be able to correct this error by just wrapping your\nexisting content in a section element.\n'),
(u'''\
<doc id='ftl2004'/>
'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\ndoc element must have a prologue.\nDiagnostic message:\nA document (the doc element) must have a prologue and one or\nmore sections. Please correct your submission by adding the\nrequired elements, then re-submit. For your records, the\nsubmission ID is ftl2004.\nAssertion failure:\ndoc element must have at least one section.\nDiagnostic message:\nA document (the doc element) must have a prologue and one or\nmore sections. Please correct your submission by adding the\nrequired elements, then re-submit. For your records, the\nsubmission ID is ftl2004.\nDiagnostic message:\nSections are sometimes omitted because authors think they can\njust place the document contents directly after the prologue.\nYou may be able to correct this error by just wrapping your\nexisting content in a section element.\n')
)
)

EXAMPLES += [EG_4_5]

EG_5_1 = (
'eg5_1',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Special XHTML conventions</title>
  <ns uri="http://www.w3.org/1999/xhtml" prefix="html"/>
  <pattern name="Document head">
    <rule context="html:head">
      <assert test="html:title">Page does not have a title.</assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link rel="stylesheet" type="text/css" href="std.css"/>
    <title>Document head</title>
  </head>
  <body class="std-body">
    <div class="std-top">Top component</div>
  </body>
</html>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Special XHTML conventions\nProcessing pattern: Document head\n'),
),
(
(u'''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link rel="stylesheet" type="text/css" href="std.css"/>
  </head>
  <body class="std-body">
    <div class="std-top">Top component</div>
  </body>
</html>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Special XHTML conventions\nProcessing pattern: Document head\nAssertion failure:\nPage does not have a title.\n'),
)
)

EXAMPLES += [EG_5_1]

EG_5_2 = (
'eg5_2',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron"
        xmlns:html="http://www.w3.org/1999/xhtml">
  <title>Technical document schema</title>
  <pattern name="Major elements">
    <rule context="doc">
      <assert test="section">
        <html:p>
          <name/> must have at least one <html:code>section</html:code>
          child.
        </html:p>
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\n'),
),
(
(u'''\
<doc>
  <prologue/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Major elements\nAssertion failure:\n<p xmlns="http://www.w3.org/1999/xhtml">\ndoc must have at least one <code>section</code>\nchild.\n</p>\n'),
)
)

EXAMPLES += [EG_5_2]

EG_5_3 = (
'eg5_3',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron"
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <title>Technical document schema</title>
  <xsl:key name="author-e-mails" match="author" use="@e-mail"/>
  <pattern name="Main contact">
    <rule context="main-contact">
      <assert test="key('author-e-mails', @e-mail)">
        "e-mail" attribute must match the e-mail of one of the authors
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
    <author member='yes' e-mail="okey.agu@navy.mil">Okechukwu Agu</author>
    <main-contact e-mail='cemereuwa@nasa.gov'/>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Main contact\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
    <author member='yes' e-mail="okey.agu@navy.mil">Okechukwu Agu</author>
    <main-contact e-mail='info@nasa.gov'/>
  </prologue>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Main contact\nAssertion failure:\n"e-mail" attribute must match the e-mail of one of the authors\n'),
)
)

EXAMPLES += [EG_5_3]

EG_5_4 = (
'eg5_4',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Technical document schema</title>
  <pattern name="Section minimum">
    <rule context="doc">
      <assert test="count(section) >= 3*count(prologue/author)">
        There must be at least three sections for each author.
      </assert>
    </rule>
  </pattern>
</schema>'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
  </prologue>
  <section/>
  <section/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Section minimum\n'),
),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">Chikezie Emereuwa</author>
  </prologue>
  <section/>
  <section/>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Section minimum\nAssertion failure:\nThere must be at least three sections for each author.\n'),
)
)

EXAMPLES += [EG_5_4]

EG_5_5 = (
'eg5_5',
u'''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron"
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <title>Technical document schema</title>
  <xsl:key name="author-e-mails" match="author" use="@e-mail"/>
  <phase id="quick-check"> <!-- "minimal sanity check" -->
    <active pattern="rightdoc" />
  </phase>
  <phase id="full-check">
    <active pattern="rightdoc" />
    <active pattern="extradoc" />
    <active pattern="majelements" />
  </phase>
  <phase id="process-links">
    <active pattern="report-link" />
  </phase>
  <pattern id="rightdoc" name="Document root">
    <rule context="/">
      <assert test="doc">Root element must be "doc".</assert>
    </rule>
  </pattern>
  <pattern id="extradoc" name="Extraneous docs">
    <rule context="doc">
      <assert test="not(ancestor::*)">
        The "doc" element is only allowed at the document root.
      </assert>
    </rule>
  </pattern>
  <pattern id="majelements" name="Major elements">
    <rule context="doc">
      <assert test="prologue">
        <name/> must have a "prologue" child.
      </assert>
      <assert test="section">
        <name/> must have at least one "section" child.
      </assert>
    </rule>
  </pattern>
  <pattern id="report-link" name="Report links">
    <rule context="*">
      <report test="@link">
        <name/> element has a link to <value-of select="@link"/>.
      </report>
    </rule>
  </pattern>
</schema>
'''.encode(ENCODING),
(
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">
      <name>Chikezie Emereuwa</name>
      <bio>Chikezie Emereuwa is a quantum engineer and NASA researcher</bio>
      <affiliation>NASA</affiliation>
    </author>
  </prologue>
  <section>
    Actual contents may have shifted in transmission.
  </section>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nProcessing pattern: Extraneous docs\nProcessing pattern: Major elements\nProcessing pattern: Report links\n'),
(u'''\
<doc>
  <prologue/>
  <section>
    Placeholder for the <emphasis link='http://nasa.gov/ftl/paper.xml'>actual content</emphasis>.
  </section>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nProcessing pattern: Extraneous docs\nProcessing pattern: Major elements\nProcessing pattern: Report links\nReport:\nemphasis element has a link to http://nasa.gov/ftl/paper.xml.\n'),
),
(
(u'''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <link rel="stylesheet" type="text/css" href="std.css"/>
    <title>Document head</title>
  </head>
  <body class="std-body">
    <div class="std-top">Top component</div>
  </body>
</html>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nAssertion failure:\nRoot element must be "doc".\nProcessing pattern: Extraneous docs\nProcessing pattern: Major elements\nProcessing pattern: Report links\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">
      <name>Chikezie Emereuwa</name>
      <bio>Chikezie Emereuwa is a quantum engineer and NASA researcher</bio>
      <affiliation>NASA</affiliation>
    </author>
  </prologue>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nProcessing pattern: Extraneous docs\nProcessing pattern: Major elements\nAssertion failure:\ndoc must have at least one "section" child.\nProcessing pattern: Report links\n'),
(u'''\
<doc>
  <section>
    Placeholder for the <emphasis link='http://nasa.gov/ftl/paper.xml'>actual content</emphasis>.
  </section>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nProcessing pattern: Extraneous docs\nProcessing pattern: Major elements\nAssertion failure:\ndoc must have a "prologue" child.\nProcessing pattern: Report links\nReport:\nemphasis element has a link to http://nasa.gov/ftl/paper.xml.\n'),
(u'''\
<doc>
  <prologue>
    <title>Faster than light travel</title>
    <subtitle>From fantasy to reality</subtitle>
    <author member='yes' e-mail="cemereuwa@nasa.gov">
      <name>Chikezie Emereuwa</name>
      <bio>Chikezie Emereuwa is a quantum engineer and NASA researcher</bio>
      <affiliation>NASA</affiliation>
    </author>
    <section>
      Where does the section element go again?
    </section>
  </prologue>
</doc>'''.encode(ENCODING),
'<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Technical document schema\nProcessing pattern: Document root\nProcessing pattern: Extraneous docs\nProcessing pattern: Major elements\nAssertion failure:\ndoc must have at least one "section" child.\nProcessing pattern: Report links\n'),
)
)

EXAMPLES += [EG_5_5]


"""
EG_5_2 = (
'eg5_2',
u'''\
'''.encode(ENCODING),
(
(u'''\
'''.encode(ENCODING),
''),
),
(
(u'''\
'''.encode(ENCODING),
''),
)
)

EXAMPLES += [EG_5_2]
"""


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestBasics(unittest.TestCase):
    def setUp(self):
        #self.isrc_factory = InputSource.DefaultFactory
        return

    def testStrons(self):
        for (name, stron, good, bad) in EXAMPLES:
            print name
            #open(os.path.join('/tmp/stron', name+'.sch'), 'w').write(stron)
            for (count, (instance, expected)) in enumerate(good):
                output = run_stron(stron, instance)
                #print output
                #normalize text, a bit slow for large files with much non-ws
                output = normalize_text(output)
                self.assertEqual(output, expected)
                #open(os.path.join('/tmp/stron', name+'_good%i.xml'%(count+1)), 'w').write(instance)
            for (count, (instance, expected)) in enumerate(bad):
                output = run_stron(stron, instance)
                #print output
                #normalize text, a bit slow for large files with much non-ws
                output = normalize_text(output)
                self.assertEqual(output, expected)
                #open(os.path.join('/tmp/stron', name+'_bad%i.xml'%(count+1)), 'w').write(instance)
        return


if __name__ == '__main__':
    unittest.main()




#! /usr/bin/env python

"""Tool for measuring execution time of Python scripts

Based on timeit.py, modified by Jeremy Kloth for some code
simplification and to work with script bodies, rather than statements.


This module avoids a number of common traps for measuring execution
times.  See also Tim Peters' introduction to the Algorithms chapter in
the Python Cookbook, published by O'Reilly.

Command line usage:
    python timer.py [-n N] [-r N] [-p] [-v] [-h] file

Options:
  -r/--repeat N: how many times to repeat the timer (default 3)
  -p/--profile: profile execution
  -c/--calls: how many functions calls to display for profiling
  -v/--verbose: print raw timing results; repeat for more digits precision
  -h/--help: print this usage message and exit
  file: Python file to be timed (must define a main() function)

For plain timings, a suitable number of loops is calculated by trying
successive powers of 10 until the total time is at least 0.2 seconds.

The difference in default timer function is because on Windows,
clock() has microsecond granularity but time()'s granularity is 1/60th
of a second; on Unix, clock() has 1/100th of a second granularity and
time() is much more precise.  On either platform, the default timer
functions measure wall clock time, not the CPU time.  This means that
other processes running on the same computer may interfere with the
timing.  The best thing to do when accurate timing is necessary is to
repeat the timing a few times and use the best time.  The -r option is
good for this; the default of 3 repetitions is probably enough in most
cases.  On Unix, you can use clock() to measure CPU time.
"""

import gc
import sys
import time
import profile
import pstats

default_repeat = 3
default_calls = 50

if sys.platform == "win32":
    # On Windows, the best timer is time.clock()
    default_timer = time.clock
else:
    # On most other platforms the best timer is time.time()
    default_timer = time.time

def main(args=None):
    if args is None:
        args = sys.argv[1:]
    import getopt
    try:
        opts, args = getopt.getopt(args, "n:r:c:pvh",
                                   ["number=", "repeat=", "calls:",
                                    "profile", "verbose", "help"])
    except getopt.error, err:
        print err
        print "use -h/--help for command line help"
        return 2

    number = None
    repeat = default_repeat
    calls = default_calls
    verbose = 0
    precision = 3
    timeit = True
    help = False

    for o, a in opts:
        if o in ("-n", "--number"):
            number = int(a)
            if number <= 0:
                number = None # auto
        if o in ("-r", "--repeat"):
            repeat = int(a)
            if repeat <= 0:
                repeat = 1
        if o in ("-c", "--calls"):
            calls = int(a)
        if o in ("-p", "--profile"):
            timeit = False
        if o in ("-v", "--verbose"):
            if verbose:
                precision += 1
            verbose += 1
        if o in ("-h", "--help"):
            help = True

    if help or not args:
        print __doc__,
        return 0

    # Include the current directory, so that local imports work (sys.path
    # contains the directory of this script, rather than the current
    # directory)
    import os
    sys.path.insert(0, os.curdir)

    script = {}
    sys.argv = args
    script = {}
    execfile(args[0], script)
    try:
        script_main = script['main']
    except KeyError:
        print "unable to execute '%s': no main() function" % args[0]
        return 1

    if timeit:
        # determine number so that 0.2 <= total time < 2.0
        if number is None:
            for i in xrange(10):
                number = 10**i
                x = default_timer()
                for i in xrange(number):
                    script_main()
                x = default_timer() - x
                if verbose:
                    print "%d loops -> %.*g secs" % (number, precision, x)
                if x >= 0.2:
                    break

        timings = []
        seq = range(number)
        for i in xrange(repeat):
            gcold = gc.isenabled()
            gc.disable()
            start = default_timer()
            for i in seq:
                script_main()
            end = default_timer()
            timings.append(end - start)
            if gcold:
                gc.enable()
            gc.collect()
        best = min(timings)
        print "%d loops," % number,
        usec = best * 1e6 / number
    else:
        profiles = []
        for i in xrange(repeat):
            p = profile.Profile()
            p.runcall(script_main)
            stats = pstats.Stats(p)
            profiles.append((stats.total_tt, stats))
        best, stats = min(profiles)
        stats.strip_dirs()
        stats.sort_stats('cum').print_stats(calls)
        usec = best * 1e6

    if usec < 1000:
        print "best of %d: %.*g usec" % (repeat, precision, usec)
    else:
        msec = usec / 1000
        if msec < 1000:
            print "best of %d: %.*g msec" % (repeat, precision, msec)
        else:
            sec = msec / 1000
            print "best of %d: %.*g sec" % (repeat, precision, sec)
    return 0

if __name__ == "__main__":
    sys.exit(main())



#IGNORE FOR NOW

#In order to use Anobind to serialize back to XML, you must mix-in
#Code that interprets Bird as an element

class Bird:
    def __init__(self, name, wingspan):
        self.name = name
        self.wingspan = wingspan

b = Bird("Golden Eagle", "230cm")




#Uses Amara to print an XML file containing the directory/file subtree within
#A given path
#See http://copia.ogbuji.net/blog/2005-05-09/XML_recurs

import os
import sys
from amara import binderytools

root = sys.argv[1]

doc = binderytools.create_document()
name = unicode(root)
doc.xml_append(
    doc.xml_element(u'directory', attributes={u'name': name})
)
dirs = {root: doc.directory}

for cdir, subdirs, files in os.walk(root):
    cdir_elem = dirs[cdir]
    name = unicode(cdir)
    for f in files:
        name = unicode(f)
        cdir_elem.xml_append(
            doc.xml_element(u'file', attributes={u'name': name})
            )
    for subdir in subdirs:
        full_subdir = os.path.join(root, subdir)
        name = unicode(full_subdir)
        subdir_elem = doc.xml_element(u'directory',
                                      attributes={u'name': name})
        cdir_elem.xml_append(subdir_elem)
        dirs[full_subdir] = subdir_elem

print doc.xml(indent=u"yes")  #Print it




"""
Examples of sax2dom_chunker in action.
Don't forget amara.domtools.pushdom, with which you could
reduce this entire file to about 10 lines
"""

from Ft.Xml import Sax
from Ft.Lib import Uri
from amara.saxtools import sax2dom_chunker

#Example 1: Print out all the cities in which someone named "Eliot" lives


SEARCH_FOR_STRING = 'eliot'

def process_label(docfrag):
    #Invoked for each label
    label = docfrag.firstChild
    name = label.xpath('string(name)')
    city = label.xpath('string(address/city)')
    if name.lower().find(SEARCH_FOR_STRING) != -1:
        print city.encode('utf-8')
    return

#The path for chunking by label element
#Equivalent to the XPattern /labels/label
LABEL_PATH = [ (None, u'labels'),  (None, u'label') ]

#Create an instance of the chunker
handler = sax2dom_chunker(
    xpatterns=['/labels/label'],
    chunk_consumer=process_label
    )

parser = Sax.CreateParser()

#The chunker is the SAX handler
parser.setContentHandler(handler)
parser.parse(Uri.OsPathToUri('../labels1.xml'))

#
#Example 2: Print out all people and their street address
#

PATHS = ['/labels/label/name', '/labels/label/address/street']

def process_chunk(docfrag):
    #Invoked for each name or address.  We're getting the leaf element
    #of the path itself (in a doc frag wrapper) so just print its text
    #content
    text = docfrag.firstChild.firstChild.data
    print text.encode('utf-8')
    return


#Create an instance of the chunker
handler = sax2dom_chunker(
    xpatterns=PATHS,
    chunk_consumer=process_chunk
    )

parser = Sax.CreateParser()

#The chunker is the SAX handler
parser.setContentHandler(handler)
parser.parse(Uri.OsPathToUri('../labels1.xml'))

#
#Example 3: Print out all the cities in which someone named "Eliot" lives
#

PROD_NS = 'http://example.com/product-info'
XHTML_NS = 'http://www.w3.org/1999/xhtml'
SEARCH_FOR_STRING = 'python'

def process_product(docfrag):
    #Invoked for each product
    product = docfrag.firstChild
    desc = product.xpath('string(.//p:description[1])',
                         explicitNss={'p': PROD_NS})
    if desc.lower().find(SEARCH_FOR_STRING) != -1:
        print product.getAttributeNS(None, u'id').encode('utf-8')
    return

#The path for chunking by label element
#Equivalent to the XPattern /products/p:product
PROD_PATH = [ u'products/p:product']

#Create an instance of the chunker
handler = sax2dom_chunker(
    xpatterns=PROD_PATH,
    nss={u'p': PROD_NS},
    chunk_consumer=process_product
    )

parser = Sax.CreateParser()

#The chunker is the SAX handler
parser.setContentHandler(handler)

parser.parse(Uri.OsPathToUri('../products.xml'))
#doc = handler.get_root_node()
#from Ft.Xml.Domlette import Print
#Print(doc)




#!/usr/bin/env python
"""
Python compiler from ISO Schematron to a Python validator script
"""

import os
import sys
import codecs
import optparse
import warnings
import cStringIO
from Ft.Xml import Sax
from amara import saxtools
from Ft.Xml.Xslt import XSL_NAMESPACE


_ = lambda x:x.encode('utf-8')

SCRIPT_HEADER = u'#!/usr/bin/env python'

TOP_SKEL = u'''\
%s
#Warning: this is an auto-generated file.  Do not edit unless you're
#sure you know what you're doing

import sys
import codecs
import optparse
import cStringIO
from Ft.Xml import InputSource, CreateInputSource
from Ft.Xml.Xslt import PatternList, parser
from Ft.Xml.Xslt import Stylesheet, Processor, OutputHandler, OutputParameters
from Ft.Xml.XPath import Compile as CompileXPath
from Ft.Xml.XPath.Context import Context as XPathContext
from Ft.Xml.Xslt.XsltContext import XsltContext
from Ft.Xml.Domlette import NonvalidatingReader, GetAllNs
from Ft.Xml.XPath import Conversions
from Ft.Xml.XPath import Util
from Ft.Xml.XPath import CoreFunctions

from Ft.Xml.Xslt.XmlWriter import XmlWriter
from Ft.Xml.Xslt.OutputParameters import OutputParameters
from Ft.Xml.Xslt import XsltFunctions, Exslt
from Ft.Lib import Uri

from amara import domtools

XPATTERN_PARSER = parser.new()
del parser

#STRON_DOC_DUMMY_BASE = 'http://4suite.org/amara/scimitar/schematron-file'

'''%SCRIPT_HEADER

QB_SKEL = u'''\
'''

MAIN_SKEL = u'''
#Set up the function library for context objects according to queryBinding attr
#Determine from the query binding whether functions could require a result tree
RESULT_TREE_PROCESSOR = False
if QUERY_BINDING == u'full':
    FUNCTIONS = XsltContext.functions.copy()
    RESULT_TREE_PROCESSOR = True
elif QUERY_BINDING == u'exslt':
    FUNCTIONS = CoreFunctions.CoreFunctions.copy()
    FUNCTIONS.update(XsltFunctions.CoreFunctions)
    FUNCTIONS.update(Exslt.ExtFunctions)
    RESULT_TREE_PROCESSOR = True
elif QUERY_BINDING == u'xslt':
    FUNCTIONS = CoreFunctions.CoreFunctions.copy()
    FUNCTIONS.update(XsltFunctions.CoreFunctions)
elif QUERY_BINDING == u'xpath':
    FUNCTIONS = CoreFunctions.CoreFunctions.copy()
else:
    raise ValueError('Unknown query binding: %%s'%%repr(QUERY_BINDING))


class faux_instruction:
    baseUri = STRON_BASE_URI

class faux_root_node:
    sources = []

class faux_xslt_proc(Stylesheet.StylesheetElement, Processor.Processor):
    #Pretends to be an XSLT processor for processing keys
    def __init__(self, doc):
        self.namespaces = NSS
        #self._keys = [ (Util.ExpandQName(k[0], doc), k[1], k[2]) for k in KEYS ]
        keys = [ (Util.ExpandQName(k[0], doc), (k[1], k[2], GetAllNs(doc))) for k in KEYS ]
        self.keys = {}
        self._keys = {}
        for name, info in keys:
            self._keys.setdefault(name, []).append(info)
        #Update all the keys for all documents in the context
        #Note: 4Suite uses lazy key eval.  Consider emulating this
        for key in self._keys:
            Stylesheet.StylesheetElement.updateKey(self, doc, key[0], self)
        if RESULT_TREE_PROCESSOR:
            Processor.Processor.__init__(self)
            #Start code from Processor.runNode
            if hasattr(doc, 'baseURI'):
                node_baseUri = doc.baseURI
            elif hasattr(doc, 'refUri'):
                node_baseUri = doc.refUri
            else:
                node_baseUri = None
            sourceUri = node_baseUri or Uri.BASIC_RESOLVER.generate()
            #Create a dummy iSrc
            docInputSource = InputSource.InputSource(
                None, sourceUri, processIncludes=1,
                stripElements=self.getStripElements(),
                factory=self.inputSourceFactory)
            #return self.execute(node, docInputSource, outputStream=None)
            outputStream = cStringIO.StringIO()
            def writer_changed_handler(newWriter):
                self.writers[-1] = newWriter
            self.outputParams = OutputParameters()
            writer = OutputHandler.OutputHandler(
                self.outputParams, outputStream, writer_changed_handler)
            self.writers = [writer]
        self.initialFunctions = {}
        self.stylesheet = self
        self.root = faux_root_node()
        self._documentInputSource = InputSource.DefaultFactory.fromString('<dummy/>', Uri.OsPathToUri('.'))
        return


def validate(xmlf, reportf, phase=None):
    global WRITER, EXTFUNCTS
    oparams = OutputParameters()
    oparams.indent = 'yes'
    WRITER = XmlWriter(oparams, reportf)
    WRITER.startDocument()

    WRITER.text(u'Processing schema: ')
    WRITER.text(%(title)s)
    WRITER.text(u'\\n\\n')
    
    if xmlf == '-':
        doc = NonvalidatingReader.parseStream(sys.stdin, 'urn:stron-candidate-dummy')
    elif not isinstance(xmlf, str):
        doc = NonvalidatingReader.parseStream(xmlf, 'urn:stron-candidate-dummy')
    else:
        try:
            doc = NonvalidatingReader.parseUri(xmlf)
        except ValueError:
            doc = NonvalidatingReader.parseUri(Uri.OsPathToUri(xmlf))

    global key_handler
    key_handler = faux_xslt_proc(doc)
    #Pre-process keys, if any
    #for kname, (kuse, kmatch) in KEYS.items:

    if phase in [None, u'#DEFAULT']:
        phase = DEFAULT_PHASE
    if phase == u'#ALL':
        patterns = PATTERNS
    else:
        patterns = [ p for p in PATTERNS if p[1] in PHASES[phase] ]

    #Main rule-processing loop
    if patterns:
        for pat in patterns:
            #if not pat[2].keys():
            #    WRITER.text(_(u'Pattern context not given'))
            plist = PatternList(pat[2].keys(), NSS)
            WRITER.text(u'Processing pattern: ')
            WRITER.text(pat[0] or u'[unnamed]')
            WRITER.text(u'\\n\\n')
            for node in domtools.doc_order_iter(doc):
                matches = plist.lookup(node)
                if matches:
                    func = pat[2][matches[0]]
                    func(node)
    else:
        #Second parameter is a dictionary of prefix to namespace mappings
        plist = PatternList(CONTEXTS.keys(), {})
        for node in domtools.doc_order_iter(doc):
            #FIXME: this is a 4Suite bug work-around.  At some point remove
            #The redundant context setting
            matches = plist.lookup(node)
            if matches:
                func = CONTEXTS[matches[0]]
                func(node)
    WRITER.endDocument()
    return


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def command_line_prep():
    from optparse import OptionParser
    usage = "%%prog [options] xml-file\\nxml-file is the XML file to be validated"
    parser = OptionParser(usage=usage)
    parser.add_option("-o", "--val-output",
                      action="store", type="string", dest="val_output",
                      help="generate the validation report file FILE (XML external parsed entity format)", metavar="FILE")
    parser.add_option("-p", "--phase",
                      action="store", type="string",
                      help="Execute the schema in the specified phase", metavar="XML_ID")
    return parser

def main(argv=[__name__]):
    #Ideas borrowed from http://www.artima.com/forums/flat.jsp?forum=106&thread=4829
    if argv is None:
        argv = sys.argv
    try:
        try:
            optparser = command_line_prep()
            global OPTIONS, ARGS
            (OPTIONS, ARGS) = optparser.parse_args(argv)
            candidate_file = ARGS[1]
        except KeyboardInterrupt:
            pass
        except:
             raise Usage(optparser.format_help())
        enc, dec, inwrap, outwrap = codecs.lookup('utf-8')
        fout = OPTIONS.val_output
        phase = OPTIONS.phase
        if fout:
            fout = open(fout, 'w')
        else:
            fout = sys.stdout
        validate(candidate_file, outwrap(fout), phase=phase)
    except Usage, err:
        print >>sys.stderr, err.msg
        return 2

'''

SCRIPT_SKEL = u'''
if __name__ == "__main__":
    sys.exit(main(sys.argv))

'''

TEST_SCRIPT_SKEL = u'''
if __name__ == "__main__":
    if not TEST:
        sys.exit(main(sys.argv))

'''

RULE_SKEL = u'''
def %(rname)s(node):
    #For context XPattern %(context)s
    vars = {}
    xpath_ctx = XsltContext(node, processor=key_handler,
                            processorNss=NSS, varBindings=vars,
                            currentNode=node
                            )
    xpath_ctx.currentInstruction = faux_instruction()
    xpath_ctx.functions = FUNCTIONS
'''

ASSERT_SKEL = u'''\
    expr = CompileXPath(%(test)s)
    if not Conversions.BooleanValue(expr.evaluate(xpath_ctx)):
'''

REPORT_SKEL = u'''\
    expr = CompileXPath(%(test)s)
    if Conversions.BooleanValue(expr.evaluate(xpath_ctx)):
'''

DIAG_SKEL = u'''\
def diag%(diagcount)i(xpath_ctx):
    #Diagnostic named %(id)s
'''

EMIT_RULE_PATTERN_ITEM = u'''\
  XPATTERN_PARSER.parse(%(pat)s): %(func)s,
'''

EMIT_DIAG_ID_ITEM = u'''\
  %(id)s: %(func)s,
'''

EMIT_PHASE_ITEM = u'''\
  %(id)s: %(patterns)s,
'''

EMIT_START_ELEM_SKEL = u'''\
%(indent)sWRITER.startElement(%(qname)s, %(ns)s)
'''

EMIT_END_ELEM_SKEL  = u'''\
%(indent)sWRITER.endElement(%(qname)s, %(ns)s)
'''

EMIT_ATTRIBUTE_SKEL  = u'''\
%(indent)sWRITER.attribute(%(qname)s, %(ns)s, %(val)s)
'''

EMIT_TEXT_SKEL  = u'''\
%(indent)sWRITER.text(%(text)s)
'''

EMIT_NAME_SKEL  = u'''\
%(indent)sexpr = CompileXPath(%(path)s)
%(indent)sname = CoreFunctions.Name(xpath_ctx, expr.evaluate(xpath_ctx))
%(indent)sWRITER.text(name)
'''

EMIT_VALUE_OF_SKEL  = u'''\
%(indent)sexpr = CompileXPath(%(select)s)
%(indent)sval = Conversions.StringValue(expr.evaluate(xpath_ctx))
%(indent)sWRITER.text(val)
'''


OLD_STRON_NS = 'http://www.ascc.net/xml/schematron'
ISO_STRON_NS = 'http://purl.oclc.org/dsdl/schematron'

#The marker within an abstract pattern template for a chunk to be parameterized
ABSPAT_DELIMITER = u'\u0001'    #A char that cannot appear in XML


class pattern:
    def __init__(self, name, id_, abstract):
        self.name = name
        self.id = id_
        self.abstract = abstract
        self.concrete_rules = {}
        self.abstract_rules = {}
        self.rule_contexts = []


class stron_consumer:
    """
    Encapsulation of a set of semi-co-routines designed to handle SAX
    Events from Schematron
    """
    def __init__(self, output):
        self.top_dispatcher = {
            (saxtools.START_ELEMENT, STRON_NS, u'schema'):
            self.handle_schema,
            }
        if STRON_NS != OLD_STRON_NS:
            self.top_dispatcher[(saxtools.START_ELEMENT, OLD_STRON_NS, u'schema')] = self.handle_obsolete_schema
        self.output = output
        self.rule_contexts = {}
        self.rule_count = 1
        self.diag_ids = {}
        self.diag_count = 1
        self.event = None
        self.schema_title = u''
        self.patterns = []
        self.abstract_patterns = {}
        self.curr_pattern = None
        self.nss = {}
        self.keys = []
        self.vars = {}
        self.phases = {}
        self.root_schema_element_count = 0
        return

    def handle_obsolete_schema(self, end_condition):
        warnings.warn('You appear to be using the obsolete Schematron namespace "%s", which is no longer supported by Scimitar.  Please use the correct namespace "%s".'%(OLD_STRON_NS, STRON_NS), DeprecationWarning)
        yield None
        return

    def handle_schema(self, end_condition):
        #Check that there is one, and only one root sch:schema element
        self.root_schema_element_count += 1
        dispatcher = {
            (saxtools.START_ELEMENT, STRON_NS, u'title'):
            self.handle_title,
            (saxtools.START_ELEMENT, STRON_NS, u'phase'):
            self.handle_phase,
            (saxtools.START_ELEMENT, STRON_NS, u'pattern'):
            self.handle_pattern,
            (saxtools.START_ELEMENT, STRON_NS, u'rule'):
            self.handle_rule,
            (saxtools.START_ELEMENT, STRON_NS, u'diagnostic'):
            self.handle_diagnostic,
            (saxtools.START_ELEMENT, STRON_NS, u'ns'):
            self.handle_ns,
            (saxtools.START_ELEMENT, OLD_STRON_NS, u'key'):
            self.handle_key,
            (saxtools.START_ELEMENT, XSL_NAMESPACE, u'key'):
            self.handle_key,
            }
        #Initial call corresponds to the start schema element
        #Update the version, if specified.  Default version is ISO
        self.version = self.params.get((None, 'schemaVersion'), u'ISO')
        self.id = self.params.get((None, 'id'))
        self.query_binding = self.params.get((None, 'queryBinding'), u'xslt')
        self.default_phase = self.params.get((None, 'defaultPhase'), u'#ALL')
        self.output.write(TOP_SKEL)
        self.output.write(u'STRON_BASE_URI = \'%s\''%self.baseUri)
        self.output.write(u'\nQUERY_BINDING = %s\n\n'%repr(self.query_binding))
        self.output.write(QB_SKEL)
        curr_gen = None
        yield None
        while not self.event == end_condition:
            #print "schema", self.event
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        #Element closed.  Wrap up
        self.output.write(u'\nPATTERNS = [\n')
        for pat in self.patterns:
            pname = pat.name; pid = pat.id; pcontexts = pat.concrete_rules
            self.output.write(u'(\n %s, %s, {\n'%(repr(pname), repr(pid)))
            for rc in pcontexts:
                self.output.write(EMIT_RULE_PATTERN_ITEM%{'pat': repr(rc), 'func': pcontexts[rc]})
            self.output.write(u'}),\n')
        self.output.write(u']\n')
        self.output.write(u'\nPHASES = {\n')
        for id in self.phases:
            self.output.write(EMIT_PHASE_ITEM%{'id': repr(id), 'patterns': repr(self.phases[id])})
        self.output.write(u'}\n')
        self.output.write(u'\nCONTEXTS = {\n')
        for rc in self.rule_contexts:
            self.output.write(EMIT_RULE_PATTERN_ITEM%{'pat': repr(rc), 'func': self.rule_contexts[rc]})
        self.output.write(u'}\n')
        self.output.write(u'\nDIAGNOSTICS = {\n')
        for id in self.diag_ids:
            self.output.write(EMIT_DIAG_ID_ITEM%{'id': repr(id), 'func': self.diag_ids[id]})
        self.output.write(u'}\n')
        self.output.write(u'\nNSS = {\n')
        for prefix in self.nss:
            ns = self.nss[prefix]
            self.output.write(u'%(prefix)s: %(ns)s, '%{'prefix': repr(prefix), 'ns': ns and repr(ns)})
        self.output.write(u'}\n')
        self.output.write(u'\nKEYS = [\n')
        for n, m, u in self.keys:
            self.output.write(u'(%(n)s, %(m)s, CompileXPath(%(u)s)),'%{'n': repr(n), 'm': repr(m), 'u': repr(u)})
        self.output.write(u']\n')
        
        self.output.write(u'\nDEFAULT_PHASE = %s\n\n'%repr(self.default_phase))
        self.output.write(BOTTOM_SKEL%{'title': repr(self.schema_title)})
        raise StopIteration

    def handle_pattern(self, end_condition):
        name = self.params.get((None, 'name'))
        id_ = self.params.get((None, 'id'))
        abstract = self.params.get((None, 'abstract')) == u'true'
        isa = self.params.get((None, 'is-a'))
        self.curr_pattern = pattern(name, id_, abstract)
        if abstract:
            save_output = self.output
            self.output = cStringIO.StringIO()
        if isa:
            params = {}
        def handle_param(end_condition):
            formal = self.params.get((None, 'formal'))
            actual = self.params.get((None, 'actual'))
            if formal is None or actual is None:
                raise ValueError(_(u"sch:param must have 'formal' and 'actual' attributes."))
            params[formal] = actual
            yield None
            return
        dispatcher = {
            (saxtools.START_ELEMENT, STRON_NS, 'rule'):
            self.handle_rule,
            (saxtools.START_ELEMENT, STRON_NS, 'param'):
            handle_param,
            }
        curr_gen = None
        yield None
        while not self.event == end_condition:
            #print "pattern", self.event
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        #Element closed.  Wrap up
        if abstract:
            #Create a closure for resolving the abstract pattern
            def resolve_abspat(
                    params, template=self.output.getvalue(),
                    rule_contexts=self.curr_pattern.rule_contexts):
                #print >> sys.stderr, 'Resolving abstract pattern', template, params
                #Must sort param keys so that e.g. given one formal param $a
                #and another $ab, processing the first doesn't break
                #processing of the second
                param_keys = params.keys()
                param_keys.sort(lambda a, b: cmp(len(b), len(a)))

                rule_contexts = iter(rule_contexts)
                chunks = template.split(ABSPAT_DELIMITER)
                for count, chunk in enumerate(chunks):
                    if count % 2:
                        if chunk == '$':
                            rfunc = u'rule' + str(self.rule_count)
                            context = rule_contexts.next()
                            for formal in param_keys:
                                context = context.replace('$'+formal, params[formal])
                            self.curr_pattern.concrete_rules[context] = rfunc
                            self.output.write(str(self.rule_count))
                            self.rule_count += 1
                        else:
                            for formal in param_keys:
                                chunk = chunk.replace('$'+formal, params[formal])
                            self.output.write(repr(chunk))
                    else:
                        self.output.write(chunk)
            self.output = save_output
            self.abstract_patterns[id_] = resolve_abspat
        if isa:
            #FIXME: assumes abs pattern is defined before the parameter pattern
            #Probably not a valid restriction
            resolve = self.abstract_patterns[isa]
            resolve(params)
        if not self.curr_pattern.abstract:
            self.patterns.append(self.curr_pattern)
        self.curr_pattern = None
        raise StopIteration

    def handle_rule(self, end_condition):
        dispatcher = {
            (saxtools.START_ELEMENT, STRON_NS, 'assert'):
            self.handle_assert,
            (saxtools.START_ELEMENT, STRON_NS, 'report'):
            self.handle_report,
            (saxtools.START_ELEMENT, STRON_NS, 'let'):
            self.handle_rule_let,
            (saxtools.START_ELEMENT, STRON_NS, 'extends'):
            self.handle_extends,
            }
        id_ = self.params.get((None, 'id'))
        context = self.params.get((None, 'context'))
        abstract = self.params.get((None, 'abstract'), 'no') == 'yes'
        if abstract and context:
            raise TypeError(_(u'An abstract rule cannot have a context'))
        if not abstract and not context:
            raise TypeError(_(u'A non-abstract rule must have a context'))
        save_vars = self.vars
        if self.curr_pattern and self.curr_pattern.abstract:
            #Handle parameterization for context in abstract patterns
            marked_context = ABSPAT_DELIMITER + context + ABSPAT_DELIMITER
            rfunc_name = u'rule' + ABSPAT_DELIMITER + '$' + ABSPAT_DELIMITER
            self.output.write(RULE_SKEL%{'rname': rfunc_name, 'context': marked_context})
            self.curr_pattern.rule_contexts.append(context)
        else:
            if id_:
                rfunc_name = u'rule_' + id_
            else:
                rfunc_name = u'rule' + str(self.rule_count)
                self.rule_count += 1
            self.output.write(
                RULE_SKEL%{'rname': rfunc_name,
                           'context': repr(context)})
            if abstract:
                self.curr_pattern.abstract_rules[id] = rfunc_name
            else:
                self.curr_pattern.rule_contexts.append(context)
                self.curr_pattern.concrete_rules[context] = rfunc_name
        curr_gen = None
        yield None
        while not self.event == end_condition:
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        #Element closed.  Wrap up
        self.output.write(u'\n    return\n\n')
        self.vars = save_vars
        raise StopIteration

    def handle_extends(self, end_condition):
        rule = self.params.get((None, 'rule'))
        self.output.write(u'    rule_' + rule + '(node)\n')
        yield None
        return

    def handle_assert(self, end_condition):
        test = self.params.get((None, 'test'))
        diagnostics = self.params.get((None, 'diagnostics'))
        #FIXME: should use NMTOKENS savvy splitter
        diagnostics = diagnostics and diagnostics.split() or []
        #We want to capture all char data and markup children, so register a
        #special handler for this purpose
        curr_gen = self.xml_repeater(' '*8)
        if self.curr_pattern and self.curr_pattern.abstract:
            #Handle parameterization for asserts in abstract patterns
            marked_test = ABSPAT_DELIMITER + test + ABSPAT_DELIMITER
            self.output.write(ASSERT_SKEL%{'test': marked_test})
        else:
            self.output.write(ASSERT_SKEL%{'test': repr(test)})
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*8, 'text': repr(u'Assertion failure:\n')})
        yield None
        while not self.event == end_condition:
            #if curr_gen: curr_gen = tenorsax.delegate(curr_gen)
            curr_gen.next()
            yield None
        #Element closed.  Wrap up
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*8, 'text': repr(u'\n')})
        for diag_id in diagnostics:
            self.output.write(u'\n%(indent)sDIAGNOSTICS[%(id)s](xpath_ctx)\n'%{'indent': ' '*8, 'id': repr(diag_id)})
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*8, 'text': repr(u'\n')})
        #del curr_gen
        raise StopIteration

    def handle_report(self, end_condition):
        test = self.params.get((None, 'test'))
        #We want to capture all char data and markup children, so register a
        #special handler for this purpose
        curr_gen = self.xml_repeater(' '*8)
        if self.curr_pattern and self.curr_pattern.abstract:
            #Handle parameterization for reports in abstract patterns
            marked_test = ABSPAT_DELIMITER + test + ABSPAT_DELIMITER
            self.output.write(REPORT_SKEL%{'test': marked_test})
        else:
            self.output.write(REPORT_SKEL%{'test': repr(test)})
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*8, 'text': repr(u'Report:\n')})
        yield None
        while not self.event == end_condition:
            curr_gen.next()
            yield None
        #Element closed.  Wrap up
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*8, 'text': repr(u'\n\n')})
        #del curr_gen
        raise StopIteration

    def handle_diagnostic(self, end_condition):
        id = self.params.get((None, 'id'))
        #We want to capture all char data and markup children, so register a
        #special handler for this purpose
        curr_gen = self.xml_repeater(' '*4)
        self.output.write(DIAG_SKEL%{'diagcount': self.diag_count, 'id': repr(id)})
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*4, 'text': repr(u'Diagnostic message:\n')})
        self.diag_ids[id] = 'diag' + str(self.diag_count)
        self.diag_count += 1
        yield None
        while not self.event == end_condition:
            curr_gen.next()
            yield None
        #Element closed.  Wrap up
        self.output.write(EMIT_TEXT_SKEL%{'indent': ' '*4, 'text': repr(u'\n')})
        self.output.write(u'\n    return\n\n')
        #del curr_gen
        raise StopIteration

    def handle_phase(self, end_condition):
        id = self.params.get((None, 'id'))
        self.phases[id] = []
        def handle_active(end_condition):
            pattern = self.params.get((None, 'pattern'))
            self.phases[id].append(pattern)
            yield None
            return
        dispatcher = {
            (saxtools.START_ELEMENT, STRON_NS, 'active'):
            handle_active,
            }
        curr_gen = None
        yield None
        while not self.event == end_condition:
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        raise StopIteration

    def handle_title(self, end_condition):
        yield None
        while not self.event == end_condition:
            if self.event[0] == (saxtools.CHARACTER_DATA):
                self.schema_title += self.params
            yield None

    def handle_key(self, end_condition):
        name = self.params.get((None, 'name'))
        use = self.params.get((None, 'use'))
        match = self.params.get((None, 'match'))
        self.keys.append((name, match, use))
        yield None
        return

    def handle_ns(self, end_condition):
        prefix = self.params.get((None, 'prefix'))
        ns = self.params.get((None, 'uri'))
        self.nss[prefix] = ns
        yield None
        return

    def handle_rule_let(self, end_condition):
        name = self.params.get((None, 'name'))
        value = self.params.get((None, 'value'))
        #self.output.write(' '*4 + 'xpath_ctx.currentInstruction = faux_instruction()\n')
        self.output.write(' '*4 + 'xpath_ctx.varBindings[(None, ' + repr(name) + ')] = CompileXPath(' + repr(value) + ').evaluate(xpath_ctx)\n')
        #self.output.write(' '*4 + 'vars[(None, ' + repr(name) + ')] = CompileXPath(' + repr(value) + ').evaluate(xpath_ctx)\n')
        #self.output.write(' '*4 + 'xpath_ctx = XsltContext(node, currentNode=node, processor=key_handler, processorNss=NSS, varBindings=vars)\n')
        #self.output.write(' '*4 + 'xpath_ctx.functions = FUNCTIONS\n')
        #self.vars[name] = value
        yield None
        return

    def xml_repeater(self, indent):
        #print "xml_repeater"
        while 1:
            if self.event[0] == saxtools.START_ELEMENT:
                #emph, name and value-of are special cases
                if self.event[1:3] == (STRON_NS, u'name'):
                    path = self.params.get((None, 'path'), u'.')
                    if self.curr_pattern and self.curr_pattern.abstract:
                        #Handle parameterization in abstract patterns
                        marked_path = ABSPAT_DELIMITER + path + ABSPAT_DELIMITER
                        self.output.write(EMIT_NAME_SKEL%{'indent': indent, 'path': marked_path})
                    else:
                        self.output.write(EMIT_NAME_SKEL%{'indent': indent, 'path': repr(path)})
                elif self.event[1:3] == (STRON_NS, u'value-of'):
                    select = self.params.get((None, 'select'))
                    if self.curr_pattern and self.curr_pattern.abstract:
                        #Handle parameterization in abstract patterns
                        marked_select = ABSPAT_DELIMITER + select + ABSPAT_DELIMITER
                        self.output.write(EMIT_VALUE_OF_SKEL%{'indent': indent, 'select': marked_select})
                    else:
                        self.output.write(EMIT_VALUE_OF_SKEL%{'indent': indent, 'select': repr(select)})
                elif self.event[1:3] == (STRON_NS, u'emph'):
                    self.output.write(EMIT_START_ELEM_SKEL%{'indent': indent, 'qname': repr(self.event[2]), 'ns': repr(None)})
                else:
                    #Using local name for qname for now
                    self.output.write(EMIT_START_ELEM_SKEL%{'indent': indent, 'qname': repr(self.event[2]), 'ns': repr(self.event[1])})
            if self.event[0] == saxtools.END_ELEMENT:
                if not self.event[1:3] in [(STRON_NS, u'name'), (STRON_NS, u'value-of')]:
                    self.output.write(EMIT_END_ELEM_SKEL%{'indent': indent, 'qname': repr(self.event[2]), 'ns': repr(self.event[1])})
            if self.event[0] == saxtools.CHARACTER_DATA:
                self.output.write(EMIT_TEXT_SKEL%{'indent': indent, 'text': repr(self.params)})
            yield None


from Ft.Xml import InputSource, CreateInputSource
from Ft.Lib import Uri, Uuid

def run(stron_src, validator_outf, legacy_ns=False, prep_for_test=0):
    global BOTTOM_SKEL, STRON_NS
    if legacy_ns:
        STRON_NS = OLD_STRON_NS
    else:
        STRON_NS = ISO_STRON_NS
    if prep_for_test:
        BOTTOM_SKEL = MAIN_SKEL + TEST_SCRIPT_SKEL
    else:
        BOTTOM_SKEL = MAIN_SKEL + SCRIPT_SKEL
    parser = Sax.CreateParser()
    consumer = stron_consumer(validator_outf)
    handler = saxtools.tenorsax(consumer)
    parser.setContentHandler(handler)
    source = CreateInputSource(stron_src)
    consumer.baseUri = source.uri
    parser.parse(source)
    if not consumer.root_schema_element_count:
        raise RuntimeError('No Schematron schema was found')
    elif consumer.root_schema_element_count > 1:
        raise RuntimeError('More than one Schematron schema was found.')
    return


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


def command_line_prep():
    from optparse import OptionParser
    usage = "%prog [options] schematron-file"
    parser = OptionParser(usage=usage)
    parser.add_option("-o", "--val-script",
                      action="store", type="string", dest="stron_script",
                      help="generate the XML validator script FILE", metavar="FILE")
    parser.add_option("--legacy-ns",
                      action="store_true", dest="legacy_ns", default=0,
                      help="Use the legacy Schematron namespace")
    parser.add_option("--test",
                      action="store_true", dest="test_ready", default=0,
                      help="generate hooks for unit tests in the validator")
    #parser.add_option("-q", "--quiet",
    #                  action="store_false", dest="verbose", default=1,
    #                  help="don't print status messages to stdout")
    return parser

        
def main(argv=None):
    if argv is None:
        argv = sys.argv
    # By default, optparse usage errors are terminated by SystemExit
    try:
        optparser = command_line_prep()
        options, args = optparser.parse_args(argv[1:])
        # Process mandatory arguments with IndexError try...except blocks
        try:
            stron_fname = args[0]
        except IndexError:
            optparser.error("Missing schematron filename")
    except SystemExit, status:
        return status

    # Perform additional setup work here before dispatching to run()
    # Detectable errors encountered here should be handled and a status
    # code of 1 should be returned. Note, this would be the default code
    # for a SystemExit exception with a string message.
    fout = options.stron_script
    if not fout:
        fout = os.path.splitext(stron_fname)[0] + '-stron.' + 'py'
    fout = codecs.open(fout, 'w', 'utf-8')

    if stron_fname == '-':
        stronf = sys.stdin
    else:
        stronf = open(stron_fname, 'r')
    run(stronf, fout, options.legacy_ns, options.test_ready)

    # A status code of 0 indicates successful completion
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))





#!/usr/bin/env python
"""
Python compiler from DTLL (ISO DSDL Part 5: Datatypes) to a Python data types script
"""

import os
import re
import sys
import codecs
import optparse
import cStringIO
from xml import sax
from amara import domtools
from Ft.Xml.Domlette import GetAllNs
from Ft.Xml.Domlette import NonvalidatingReader

from xml.dom import EMPTY_NAMESPACE as NULL_NAMESPACE
from xml.dom import EMPTY_PREFIX as NULL_PREFIX

DTLL_NS = "http://www.jenitennison.com/datatypes"
WXSDT_NS = "http://www.w3.org/2001/XMLSchema-datatypes"

#FIXME: Use 4Suite L10N
def _(t): return t
NAMED_PATTERN_PAT = re.compile(r'\(\?\[(\w+)\]')


TOP_SKEL = u'''\
#Warning: this is an auto-generated file.  Do not edit unless you're
#sure you know what you're doing

import sys
import re
import codecs
'''

MAIN_SKEL = u'''\
BASE_URI = '%(ns)s'
'''

DT_CLASS_SKEL = u'''\
class %s(name)Type:
    self._name = %(name)s
'''

DT_REGEX_SKEL = u'''\
    regex = re.compile(%(regex)s)
    def __init__(self, value):
        m = self.regex.match(value)
        if not m:
            raise ValueError('Value does not conform to specified regex for data type %%s'%%(self._name))
        #Extract named patterns
        self.__dict__.update(m.groupdict())
        return
'''

DT_NO_REGEX_SKEL = u'''\
    def __init__(self, value):
        return
'''


class dtll_processor:
    def __init__(self, output_stem):
        self.reset()
        self.output_stem = output_stem
        return

    def reset(self):
        self.prefixes = {'dtll': DTLL_NS, 'wxs': WXSDT_NS}
        #Maps each data type namespace to one module of Python output
        self.outputs = {}
        return

    def execute(self, dtlldoc):
        for datatype in domtools.get_elements_by_tag_name_ns(dtlldoc, DTLL_NS, u'datatype'):
            self.handle_datatype(datatype)
        return

    def write_files(self):
        module_count = 1
        for ns, cstring in self.outputs.items():
            fout = open(self.output_stem + str(module_count) + '.py', 'w')
            fout.write(cstring.getvalue())
            module_count += 1
        return

    def handle_datatype(self, datatype):
        qname = datatype.getAttributeNS(NULL_PREFIX, u'name')
        prefix = qname[:qname.find(':') + 1][:-1] or NULL_PREFIX
        local = qname[qname.find(':')+1:]
        namespace = None
        if prefix:
            #Specified data type namespace by using a qname
            namespace = self.prefixes.get(prefix)
        if not namespace:
            #Specified data type namespace by in-scope namespaces
            namespace = GetAllNs(datatype)[prefix]
        output = self.outputs.setdefault(namespace, cStringIO.StringIO())
        output.write(TOP_SKEL)
        skel_params = {'ns': namespace}
        output.write(MAIN_SKEL%skel_params)
        skel_params = {'name': local}
        output.write(DT_CLASS_SKEL%skel_params)
        
        for parse in domtools.get_elements_by_tag_name_ns(datatype, DTLL_NS, u'parse'):
            self.handle_parse(parse, output)
        return
    
    def handle_parse(self, parse, output):
        regexen = list(domtools.get_elements_by_tag_name_ns(parse, DTLL_NS, u'regex'))
        if regexen:
            regex = python_regex(domtools.string_value(regexen[0]))
            skel_params = {'regex': regex}
            output.write(DT_REGEX_SKEL%skel_params)
        else:
            output.write(DT_NO_REGEX_SKEL)
        return
    

def python_regex(dtllregex):
    '''
    Convert a DTLL regex to a Python/Perl regex
    '''
    return NAMED_PATTERN_PAT.subn(lambda m: '(?P<'+m.group(1)+'>', dtllregex)[0]


def run(dtll_doc, output_stem, prep_for_test=0):
    #global BOTTOM_SKEL
    #if prep_for_test:
    #    BOTTOM_SKEL = MAIN_SKEL + TEST_SCRIPT_SKEL
    #else:
    #    BOTTOM_SKEL = MAIN_SKEL
    proc = dtll_processor(output_stem)
    proc.execute(dtll_doc)
    proc.write_files()
    return


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


def command_line(args):
    from optparse import OptionParser
    usage = "%prog [options] dtll-file"
    parser = OptionParser(usage=usage)
    parser.add_option("-o", "--dt-module-prefix",
                      action="store", type="string", dest="dt_modname_stem",
                      help="file name prefix for data type modules to be generated", metavar="FILE")
    parser.add_option("--test",
                      action="store_true", dest="test_ready", default=0,
                      help="generate hooks for unit tests in the data type")
    #parser.add_option("-q", "--quiet",
    #                  action="store_false", dest="verbose", default=1,
    #                  help="don't print status messages to stdout")
    global OPTIONS, ARGS
    (OPTIONS, ARGS) = parser.parse_args(args)
    return parser

        
def main(argv=None):
    #Ideas borrowed from
    # http://www.artima.com/forums/flat.jsp?forum=106&thread=4829
    if argv is None:
        argv = sys.argv
    try:
        try:
            optparser = command_line(argv)
            dtll_fname = ARGS[1]
        except KeyboardInterrupt:
            pass
        except:
             raise Usage(optparser.format_help())
        enc, dec, inwrap, outwrap = codecs.lookup('utf-8')
        output_stem = OPTIONS.dt_modname_stem
        if not output_stem:
            output_stem = os.path.splitext(dtll_fname)[0] + '-datatypes'
        if dtll_fname == '-':
            dtllf = sys.stdin
        else:
            dtllf = open(dtll_fname, 'r')
        dtll_doc = NonvalidatingReader.parseStream(dtllf, 'http://example.com')
        run(dtll_doc, output_stem, OPTIONS.test_ready)
    except Usage, err:
        print >>sys.stderr, err.msg
        return 2


if __name__ == "__main__":
    sys.exit(main(sys.argv))




#Tests XSLT transforms on bindery nodes

import unittest
import amara
from amara import binderytools

from xpath import LABELS, XBEL, RSS

IDENTITY = """\
<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
>
  <xsl:output method="xml" omit-xml-declaration="yes"/>
  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>"""

USING_PARAMS = """<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0">

  <xsl:param name='override' select='"abc"'/>
  <xsl:param name='list' select='foo'/>

  <xsl:template match="/">
    <doc>
      <overridden><xsl:value-of select='$override'/></overridden>
      <list><xsl:apply-templates select="$list"/></list>
    </doc>
  </xsl:template>

  <xsl:template match="text()">
    <item><xsl:value-of select="."/></item>
  </xsl:template>

</xsl:stylesheet>
"""


import sys

class TestXslt(unittest.TestCase):
    #def setUp(self):
    #    return

    def testIdentity(self):
        rules = [binderytools.ws_strip_element_rule(u'/*')]
        if sys.hexversion < 0x2050000: return #FIXME Amara's doc order index impl is badly incomplete.  Just happens to manifest in Python 2.4
        for source in [LABELS, XBEL]:
            doc = amara.parse(source, rules=rules)
            firstpass = doc.xml_xslt(IDENTITY)
            doc = amara.parse(firstpass, rules=rules)
            self.assertEqual(doc.xml_xslt(IDENTITY), firstpass)
        return

    def testWithParams(self):
        expected = """<?xml version="1.0" encoding="UTF-8"?>
<doc><overridden>xyz</overridden><list><item>a</item><item>b</item><item>c</item></list></doc>"""
        doc = amara.parse('<dummy/>')
        result = doc.xml_xslt(
            USING_PARAMS,
            params={'override' : 'xyz', 'list' : ['a', 'b', 'c']})
        self.assertEqual(expected, result)
        return


if __name__ == '__main__':
    unittest.main()




#Tests amara.pushbind

import unittest
import cStringIO
from xml.dom import Node

import amara

XMLDECL = '<?xml version="1.0" encoding="UTF-8"?>\n'

CONTAINER = """\
<top>
  <container id='otu'>
    <value>
      <title>kedu ka i mere</title>
      <body>Ofe nsala na atu uto</body>
    </value>
  </container>
  <container id='abuo'>
    <value>
      <title>o di mma</title>
      <body>Papa nyem ugwom, O!</body>
    </value>
  </container>
</top>
"""

class TestPushBindEventHook(unittest.TestCase):

    #Disabled until Saxlette synchronicity issues are tackled
    def testContainerExample(self):
        class container_tracker:
            def startElementNS(self, name, qname, attribs):
                if name[1] == u'container':
                    #print attribs.get((None, u'id'))
                    self.id = attribs.get((None, u'id'))

        context = container_tracker()
        check = []
        for node in amara.pushbind(CONTAINER, u'value', event_hook=context):
            #print repr(node)
            #print unicode(node.title)
            check.append((context.id, unicode(node.title)))

        EXPECTED = [(u'otu', u'kedu ka i mere'), (u'abuo', u'o di mma')]
        self.assertEqual(check, EXPECTED)
        return


if __name__ == '__main__':
    unittest.main()




"""
Based on this cry for help:
http://mail.python.org/pipermail/python-list/2006-January/323506.html
This is atrocious XML, but we've all had to deal with a mess of generated
records elements, so let's use pushbind to turn it into a nice dict
"""

XML = """\
<?xml version='1.0' ?>
<Services>
  <Service>
    <Name>name</Name>
    <Label>label</Label>
    <Icon>/opt/OV/www/htdocs/ito_op/images/service.32.gif</Icon>
    <Status>
      <Normal/>
    </Status>
    <Depth>1</Depth>
    <Attribute>
     <Name>Attrib1</Name>
     <Value>Val1</Value>
    </Attribute>
    <Attribute>
     <Name>Attrib1.1</Name>
     <Value>Val1.1</Value>
    </Attribute>
  </Service>
  <Service>
    <Name>name1</Name>
    <Label>label1</Label>
    <Icon>/opt/OV/www/htdocs/ito_op/images/service.32.gif</Icon>
    <Status>
      <Normal/>
    </Status>
    <Depth>1</Depth>
    <Attribute>
     <Name>Attrib2</Name>
     <Value>val 2</Value>
    </Attribute>
  </Service>
  <Service>
    <Name>name2</Name>
    <Label>label2</Label>
    <Icon>/opt/OV/www/htdocs/ito_op/images/service.32.gif</Icon>
    <Status>
      <Normal/>
    </Status>
    <Depth>1</Depth>
  </Service>
</Services>
"""

import amara
services = {}
for service in amara.pushbind(XML, u'Service'):
    service_attribs = {}
    if hasattr(service, 'Attribute'):
        #Using look-before-you-leap.  Could also try/except AttributeError
        #Or even XPath: for attr in service.xml_xpath(u'Attribute'):
        for attr in service.Attribute:
            service_attribs[unicode(attr.Name)] = unicode(attr.Value)
    services[unicode(service.Name)] = service_attribs

print services



#Tests bindery rules

import unittest
import datetime
import amara
from amara import binderytools

LABELS = """\
<?xml version="1.0" encoding="iso-8859-1"?>
<labels>
  <label id='ep' added="2003-06-10">
    <name>Ezra Pound</name>
    <address>
      <street>45 Usura Place</street>
      <city>Hailey</city>
      <province>ID</province>
    </address>
  </label>
  <label id='tse' added="2003-06-20">
    <name>Thomas Eliot</name>
    <address>
      <street>3 Prufrock Lane</street>
      <city>Stamford</city>
      <province>CT</province>
    </address>
  </label>
  <label id="lh" added="2004-11-01">
    <name>Langston Hughes</name>
    <address>
      <street>10 Bridge Tunnel</street>
      <city>Harlem</city>
      <province>NY</province>
    </address>
  </label>
  <label id="co" added="2004-11-15">
    <name>Christopher Okigbo</name>
    <address>
      <street>7 Heaven's Gate</street>
      <city>Idoto</city>
      <province>Anambra</province>
    </address>
  </label>
</labels>
"""

XBEL = """\
<?xml version="1.0" encoding="utf-8"?>
<!--
<!DOCTYPE xbel PUBLIC "+//IDN python.org//DTD XML Bookmark Exchange Language 1.0//EN//XML" "http://www.python.org/topics/xml/dtds/xbel-1.0.dtd">
-->
<xbel>
  <title>Some 4Suite Bookmarks</title>
  <!-- No top level bookmars -->
  <folder>
    <title>Main links</title>
    <bookmark href="http://4suite.org/">
      <title>4Suite home page</title>
    </bookmark>
    <bookmark href="http://foursuite.sourceforge.net/">
      <title>4Suite on SourceForge</title>
    </bookmark>
    <bookmark href="http://uche.ogbuji.net/tech/4Suite/">
      <title>Uche's 4Suite page</title>
    </bookmark>
    <bookmark href="http://twistedmatrix.com/users/jh.twistd/xml-sig/moin.cgi/FtSuiteWiki">
      <title>4Suite Wiki</title>
    </bookmark>
  </folder>
  <folder>
    <title>Articles</title>
    <bookmark href="http://www.xml.com/pub/a/2002/10/16/py-xml.html">
      <title>Python &amp; XML: A Tour of 4Suite</title>
    </bookmark>
    <folder>
      <title>Free registration required</title>
      <bookmark href="http://www-105.ibm.com/developerworks/education.nsf/xml-onlinecourse-bytitle/BE1A7E60838F9F7686256AF400523C58">
        <title>Python and XML development using 4Suite, Part 2: 4XPath and 4XSLT </title>
      </bookmark>
      <bookmark href="http://www-105.ibm.com/developerworks/education.nsf/xml-onlinecourse-bytitle/8A1EA5A2CF4621C386256BBB006F4CEC">
        <title>Python and XML development using 4Suite, Part 3: 4RDF</title>
      </bookmark>
      <bookmark href="http://www-105.ibm.com/developerworks/education.nsf/xml-onlinecourse-bytitle/CD1A8865668D9A2E86256C910062D445?OpenDocument">
        <title>Develop Python/XML with 4Suite, Part 5: The Repository Features</title>
      </bookmark>
    </folder>
  </folder>
  <folder>
    <title>Mailing lists</title>
    <bookmark href="http://lists.fourthought.com/mailman/listinfo/4suite">
      <title>4Suite Mailing List</title>
    </bookmark>
    <bookmark href="http://lists.fourthought.com/mailman/listinfo/4suite-ann">
      <title>4Suite Announcements Mailing List</title>
    </bookmark>
    <bookmark href="http://lists.fourthought.com/mailman/listinfo/4suite-dev">
      <title>4Suite Developer Mailing List</title>
    </bookmark>
  </folder>
</xbel>
"""

TYPE_MIX = """\
<?xml version="1.0" encoding="utf-8"?>
<a a1="1">
  <b b1="2.1"/>
  <b>3</b>
  <c c1="2005-01-31">
    <d>5</d>
    <e>2003-01-30T17:48:07.848769Z</e>
    <!--e>2003-01-30T17:48:07.848769</e--> <!-- Doesn't work right now without any TZ info (e.g. 'Z') -->
  </c>
  <g>good</g>
</a>
"""

class TestRules(unittest.TestCase):
    #def setUp(self):
    #    return

    def testMultipleSimpleStringRules(self):
        #patterns = [u'name', u'address/street', u'address/city',
        #            '/labels/label/address/province']
        #rules = [ binderytools.simple_string_element_rule(p) for p in patterns ]
        patterns = [u'bookmark/title', u'xbel/title']
        rules = [ binderytools.simple_string_element_rule(p) for p in patterns ]
        doc = amara.parse(XBEL, rules=rules)
        self.assert_(isinstance(doc.xbel.title, unicode))
        self.assert_(isinstance(doc.xbel.folder.bookmark.title, unicode))
        self.assert_(not(isinstance(doc.xbel.folder.title, unicode)))
        self.assertEqual(doc.xbel.title, u'Some 4Suite Bookmarks')
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testMultipleOmitElementRules(self):
        patterns = [u'folder/title', u'xbel/title']
        rules = [ binderytools.omit_element_rule(p) for p in patterns ]
        doc = amara.parse(XBEL, rules=rules)
        self.assertEquals(unicode(doc.xbel.folder.bookmark.title), u"4Suite home page")
        #Yeah, could just do assert_(hasattr but this way I remember how assertRaises works
        #Consider assertRaisesEx from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/307970
        self.assertRaises(AttributeError, lambda x=None: doc.xbel.title)
        self.assertRaises(AttributeError, lambda x=None: doc.xbel.folder.title)
        return

    def testSimpleAndOmitRules(self):
        rules = [
            binderytools.omit_element_rule(u'folder/title'),
            binderytools.omit_element_rule(u'xbel/title'),
            binderytools.simple_string_element_rule(u'bookmark/title')
            ]
        doc = amara.parse(XBEL, rules=rules)
        self.assert_(isinstance(doc.xbel.folder.bookmark.title, unicode))
        self.assertEquals(doc.xbel.folder.bookmark.title, u"4Suite home page")
        #Yeah, could just do assert_(hasattr but this way I remember how assertRaises works
        #Consider assertRaisesEx from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/307970
        self.assertRaises(AttributeError, lambda x=None: doc.xbel.title)
        self.assertRaises(AttributeError, lambda x=None: doc.xbel.folder.title)
        return

    def testTypeInference(self):
        rules = [ binderytools.type_inference(u"*") ]
        doc = amara.parse(TYPE_MIX, rules=rules)
        self.assertEqual(doc.a.a1, 1)
        self.assertEqual(doc.a.b.b1, 2.1)
        try:
            from dateutil.tz import tzlocal
            #self.assertEqual(doc.a.c.c1, datetime.datetime.now(tzlocal()))
            self.assertEqual(doc.a.c.c1, datetime.datetime(2005, 1, 31, tzinfo=tzlocal()))
            #Because of timezone conversion with have to check within 24 hrs (86400 seconds)
            self.assert_(doc.a.c.e - datetime.datetime(2003, 1, 30, 17, 48, 7, 848769, tzinfo=tzlocal()) < datetime.timedelta(0, 86400))
        except ImportError:
            self.assertEqual(doc.a.c.c1, datetime.datetime(2005, 1, 31))
            #Because of timezone conversion with have to check within 24 hrs (86400 seconds)
            self.assert_(doc.a.c.e - datetime.datetime(2003, 1, 30, 17, 48, 7, 848769) < datetime.timedelta(0, 86400))
        #self.assertEqual(len(list(doc.labels)), 1)
        #self.assertEqual(doc.xml_xpath(u"count(//@id)"), 4.0)
        return

    def testWhitespaceStripping1(self):
        rules = [ binderytools.ws_strip_element_rule(u"*") ]
        doc = amara.parse("<a>  <b/>  </a>", rules=rules)
        #print [ l.id for l in doc.labels.label ]
        #self.assertEqual(len(list(doc.labels.label)), 4)
        self.assertEqual(len(doc.a.xml_children), 1)
        #self.assertEqual(doc.xml_xpath(u"count(//@id)"), 4.0)
        return

    def testWhitespaceStripping4(self):
        rules = [ binderytools.ws_strip_element_rule(u"*") ]
        doc = amara.parse(LABELS, rules=rules)
        #print [ l.id for l in doc.labels.label ]
        #self.assertEqual(len(list(doc.labels.label)), 4)
        self.assertEqual(len(doc.labels.xml_children), 4)
        #self.assertEqual(doc.xml_xpath(u"count(//@id)"), 4.0)
        return

    def testElementSkeletonRule1(self):
        rules = [
            binderytools.element_skeleton_rule(u'label'),
            ]
        doc = amara.parse(LABELS, rules=rules)
        self.assertEquals(len(doc.labels.label.name.childNodes), 0)
        self.assertEquals(len(doc.labels.label[1].name.childNodes), 0)
        self.assertEquals(len(doc.labels.label[2].name.childNodes), 0)
        self.assertEquals(len(doc.labels.label.address.street.childNodes), 0)
        self.assertEquals(len(doc.labels.label[1].address.street.childNodes), 0)
        self.assertEquals(len(doc.labels.label[2].address.street.childNodes), 0)
        self.assertEquals(len(doc.labels.label.address.city.childNodes), 0)
        return

    def testElementSkeletonRule2(self):
        rules = [
            binderytools.element_skeleton_rule(u'folder'),
            ]
        doc = amara.parse(XBEL, rules=rules)
        self.assertEquals(len(doc.xbel.title.childNodes), 1)
        self.assertEquals(len(doc.xbel.folder.title.childNodes), 0)
        self.assertEquals(len(doc.xbel.folder[1].title.childNodes), 0)
        self.assertEquals(len(doc.xbel.folder[1].folder.title.childNodes), 0)
        return

    def testOmitNodeTypeRule1(self):
        from xml.dom import Node
        rules = [
            binderytools.omit_nodetype_rule(Node.COMMENT_NODE),
            ]
        XML = '<a><!-- example --><b>blablabla</b></a>'
        doc = amara.parse(XML, rules=rules)
        self.assertEqual(len(doc.a.xml_children), 1)
        return

    def testOmitNodeTypeRule2(self):
        from xml.dom import Node
        rules = [
            binderytools.omit_nodetype_rule(Node.PROCESSING_INSTRUCTION_NODE),
            ]
        XML = '<a><?target data?><b>blablabla</b></a>'
        doc = amara.parse(XML, rules=rules)
        self.assertEqual(len(doc.a.xml_children), 1)
        return

    def testOmitNodeTypeRule3(self):
        from xml.dom import Node
        rules = [
            binderytools.omit_nodetype_rule(Node.TEXT_NODE),
            ]
        XML = '<a><!-- example --><b>blablabla</b></a>'
        doc = amara.parse(XML, rules=rules)
        self.assertEqual(len(doc.a.b.xml_children), 0)
        return

    def testOmitNodeTypeRule4(self):
        from xml.dom import Node
        rules = [
            binderytools.omit_nodetype_rule(Node.COMMENT_NODE),
            binderytools.omit_nodetype_rule(Node.PROCESSING_INSTRUCTION_NODE),
            ]
        XML = '<a><?target data?><!-- example --><b>blablabla</b></a>'
        doc = amara.parse(XML, rules=rules)
        self.assertEqual(len(doc.a.xml_children), 1)
        return

    def testOmitNodeTypeRule5(self):
        from xml.dom import Node
        rules = [
            binderytools.omit_nodetype_rule(Node.COMMENT_NODE),
            ]
        XML = '<a><?target data?><!-- example --><b>blablabla</b></a>'
        doc = amara.parse(XML, rules=rules)
        self.assertEqual(len(doc.a.xml_children), 2)
        return


if __name__ == '__main__':
    unittest.main()




#Actual demo starts 12 lines below...
import os, sys
#Make sure we can find needed source files
if not os.access('monty.xml', os.F_OK):
    os.chdir('..')
    if not os.access('monty.xml', os.F_OK):
        print "This file is meant to be run from the demo directory (e.g. same directory as 'monty.xml' or in the demo/bindery directory"
        sys.exit(1)

#Actual demo starts here:
import amara
from amara import binderytools
doc = amara.parse('monty.xml')
print doc.monty.python.spam
print doc.monty.python[1]

#

doc = amara.parse('xbel.xml')
print doc.xbel.folder.bookmark.title
print doc.xbel.folder.bookmark[1].title

print doc.xbel.folder.bookmark.title
print doc.xbel.folder.bookmark[0].title
print unicode(doc.xbel.folder.bookmark.title)
print unicode(doc.xbel.folder.bookmark[0].title)

"""Cut from manual, but still demoed here:
There is a subtelty in that the first two forms are actually passing
Bindery objects to print.  Bindery objects know how to print themselves,
so you don't generally notice this.  The last four lines on the other
hand pass Python Unicode objects to print.  If you want to get a sense of
Bindery under the covers, Use the repr() function to wrap the values being
printed.
"""

print repr(doc.xbel.folder.bookmark.title)
print repr(doc.xbel.folder.bookmark[0].title)
print repr(unicode(doc.xbel.folder.bookmark.title))
print repr(unicode(doc.xbel.folder.bookmark[0].title))

print unicode(doc.xbel.folder.bookmark.title).encode('utf-8')

#
#Print all bookmark hrefs
#

def all_titles_in_folder(folder):
    for bookmark in folder.bookmark:
            print bookmark.href
    if hasattr(folder, "folder"):
        #There are sub-folders
        for folder in folder.folder:
            all_titles_in_folder(folder)
    return

for folder in doc.xbel.folder:
    all_titles_in_folder(folder)

print doc.xml() #write back out as XML to the console
print doc.xbel.folder.xml() #Just the first folder
doc.xml(sys.stdout) #write back out as XML to the console

from Ft.Xml.Xslt.XmlWriter import XmlWriter
from Ft.Xml.Xslt.OutputParameters import OutputParameters

oparams = OutputParameters()
oparams.indent = 'yes'
oparams.encoding = 'iso-8859-1'
writer = XmlWriter(oparams, sys.stdout)

#If you're serializing anything but a full document, you must
#explicitly call writer.startDocument() ... writer.endDocument()
doc.xml(writer=writer)

#
# XPath
#

print
tl_folders = doc.xbel.xml_xpath(u'folder')
for folder in tl_folders:
    print unicode(folder.title)

r = doc.xbel.folder.bookmark[0].xml_xpath(u'*[1]')
r = doc.xbel.folder.bookmark.xml_xpath(u'*[1]')
r = doc.xbel.xml_xpath(u'folder[1]/bookmark[1]/*[1]')
r = doc.xbel.xml_xpath(u'/folder[1]/bookmark[1]/*[1]')
r = doc.xml_xpath(u'xbel/folder[1]/bookmark[1]/*[1]')

print r[0]

bookmarks = doc.xml_xpath(u'//bookmark')
for bookmark in bookmarks:
    print bookmark.href

hrefs = doc.xml_xpath(u'//@href')
for href in hrefs:
    print unicode(href)

url = u"http://4suite.org/"

#
# Namespaces
#

#Set up customary namespace bindings for RSS
#These are used in XPath query and XPattern rules
RSS10_NSS = {
    u'rdf': u'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
    u'dc': u'http://purl.org/dc/elements/1.1/',
    u'rss': u'http://purl.org/rss/1.0/',
    }

doc = amara.parse('rss10.rdf', prefixes=RSS10_NSS)

#Create a dictionary of RSS items
items = {}
item_list = doc.xml_xpath(u'//rss:item')
items = dict( [ ( item.about, item) for item in item_list ] )
print items

for channel in doc.RDF.channel:
    print "Channel:", channel.about
    print "Title:", channel.title
    print "Items:"
    for item_ref in channel.items.Seq.li:
        item = items[item_ref.resource]
        print "\t", item.link
        print "\t", item.title

#Show the namespace particulars of the rdf:RDF element
print doc.RDF.namespaceURI
print doc.RDF.localName
print doc.RDF.prefix

#Get the RSS item with a given URL
item_url = u'http://www.oreillynet.com/cs/weblog/view/wlg/532'
matching_items = doc.xml_xpath(u'//rss:item[@rdf:about="%s"]'%item_url)
print matching_items
assert matching_items[0].about == item_url

#
# Modification
#

doc = amara.parse('monty.xml')
doc.monty.foo = u'bar'

doc.monty.python.xml_clear()
print doc.xml()

doc.monty.python.xml_append(doc.xml_element(None, u'new'))
doc.monty.python.new.xml_children.append(u'New Content')
print doc.xml()

print doc.monty.xml_children
ix = doc.monty.python.xml_index_on_parent()  #ix = 2
print ix
ix = doc.monty.python[1].xml_index_on_parent()  #ix = 5
print ix
doc.monty.xml_remove_child(ix)
print doc.xml()

#
# Customizing the binding
#

#
# Treating some elements as simple string values
#

import amara
from amara import binderytools
from amara import bindery

#Specify (using XPatterns) elements to be treated similarly to attributes
rules = [
    binderytools.simple_string_element_rule(u'title')
    ]
#Execute the binding
doc = amara.parse('xbel.xml', rules=rules)

#Print the title
print doc.xbel.folder.bookmark.title
print repr(doc.xbel.folder.bookmark.title)

#Same operation, but different approach

#Explicitly create a binder instance, in order to customize rules
binder = bindery.binder()
custom_rule = binderytools.simple_string_element_rule(u'title')
binder.add_rule(custom_rule)

#Execute the binding
doc = amara.parse('xbel.xml', binderobj=binder)

#Print the title
print doc.xbel.folder.bookmark.title
print repr(doc.xbel.folder.bookmark.title)


#
# Omitting some elements altogether
#

import amara
from amara import binderytools
from amara import bindery
#Explicitly create a binder instance, in order to customize rules
binder = bindery.binder()

#Specify (using XPatterns) elements to be omitted
rules = [
    binderytools.omit_element_rule(u"folder/title")
    ]
#Execute the binding
doc = amara.parse('xbel.xml', rules=rules)

#Folder titlees should be missing
try:
    print doc.xbel.folder.title
except AttributeError:
    print "AttributeError accessing doc.xbel.folder.title, as expected"
#But bookmark title should be there
print doc.xbel.folder.bookmark.title
print repr(doc.xbel.folder.bookmark.title)
doc.xml(sys.stdout)
print
print

#
# White-space stripping
#

import amara
from amara import binderytools
from amara import bindery
#Explicitly create a binder instance, in order to customize rules
binder = bindery.binder()

#Specify (using XPatterns) elements to be omitted
rules = [
    binderytools.ws_strip_element_rule(u"*")
    ]
#Execute the binding
doc = amara.parse('xbel.xml', rules=rules)

#doc.xbel.folder.title
doc.xml(sys.stdout)


#
# More detail-oriented binding API
#

import amara
from amara import binderytools
from amara import bindery
from Ft.Xml import InputSource
from Ft.Lib import Uri

#Create an input source for the XML
isrc_factory = InputSource.DefaultFactory
#Create a URI from a filename the right way
file_uri = Uri.OsPathToUri("xbel.xml", attemptAbsolute=1)
isrc = isrc_factory.fromUri(file_uri)

#Now bind from the XML given in the input source
binder = bindery.binder()

#Specify (using XPatterns) elements to be treated similarly to attributes
custom_rule = binderytools.simple_string_element_rule(u'title')
binder.add_rule(custom_rule)

#Bind
doc = binder.read_xml(isrc)

print isinstance(doc.xbel.folder.bookmark.title, unicode)
print doc.xbel.folder.bookmark.title


#
# Use your own class for binding a given element
#

import urllib
from xml.dom import Node
from amara import binderytools
from amara import bindery
from Ft.Xml import InputSource
from Ft.Lib import Uri

#Subclass from the default binding class
#We're adding a specialized method for accessing a bookmark on the net
class specialized_bookmark(bindery.element_base):
    def retrieve(self):
        try:
            stream = urllib.urlopen(self.href)
            content = stream.read()
            stream.close()
            return content
        except IOError:
            import sys; sys.stderr.write("Unable to access %s\n"%self.href)

#Explicitly create a binder instance, in order to customize rules
binder = bindery.binder()

#associate specialized_bookmark class with elements not in an XML
#namespace and having a GI of "bookmark"
binder.set_binding_class(None, "bookmark", specialized_bookmark)

#Execute the binding
doc = amara.parse('xbel.xml', binderobj=binder)

#Show specialized instance
print doc.xbel.folder.bookmark.__class__

#Exercise the custom method
print "Content of first bookmark:"
print doc.xbel.folder.bookmark.retrieve()

print "="*72
print "RSS 1.0"
print "="*72

#
# Namespaces in XPattern-customized bindings
#

#Set up customary namespace bindings for RSS
#These are used in XPath query and XPattern rules
RSS10_NSS = {
    'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
    'dc': 'http://purl.org/dc/elements/1.1/',
    'rss': 'http://purl.org/rss/1.0/',
    }

#Treat rss title, and link elements, and all DC elements as simple attributes
#BTW, simplify = [u"rss:title|rss:link|dc:*"] would work the same
simplify = [u"rss:title", u"rss:link", u"dc:*"]
custom_rule = binderytools.simple_string_element_rule(simplify)

doc = amara.parse('rss10.rdf', rules=[custom_rule], prefixes=RSS10_NSS)

#Create a dictionary of RSS items
items = {}
item_list = doc.xml_xpath(u'//rss:item')
for item in item_list:
    items[item.about] = item
print items

for channel in doc.RDF.channel:
    print "Channel:", channel.about
    print "Title:", channel.title
    print "Items:"
    for item_ref in channel.items.Seq.li:
        item = items[item_ref.resource]
        print "\t", item.link
        print "\t", item.title

#Show the namespace particulars of the rdf:RDF element
print doc.RDF.channel.title
print doc.RDF.channel.title.__class__ #Should be UNicode

for folder in amara.pushbind('xbel.xml', u'/xbel/folder'):
    title = folder.title
    bm_count = len(list(folder.bookmark))
    print "Folder", title, "has", bm_count, "top level bookmarks"





#Convert a simple, matrix-like XML format to a Python data structure
DOC = """\
<matrix>
<a><b>x1</b><c>y1</c></a>
<a><b>x2</b><c>y2</c></a>
<a><b>x3</b><c>y3</c></a>
</matrix>
"""

from amara import binderytools

matrix = []
for row in binderytools.pushbind(u'a', string=DOC):
    matrix.append((unicode(row.b), unicode(row.c)))

print matrix


matrix = []
for row in binderytools.pushbind(u'a', string=DOC):
    matrix.append(tuple([ unicode(e) for e in row.xml_xpath(u'*') ]))

print matrix




#Tests interesting patterns of XML documents

import unittest
import amara
import keyword


ROOT_MIXED_1 = "<A>x<B/>y</A>"

ROOT_MIXED_2 = """\
<A>x
  <B>Two</B>
y</A>
"""

ROOT_GI_IN_CHILD = """\
<A>
  <A>One</A>
  <B>Two</B>
</A>
"""

ROOT_GI_IN_CHILD_GRANDCHILD = """\
<A>
  <A>
    <A>A One</A>
    <B>A Two</B>
  </A>
  <B>
    <A>B One</A>
    <B>B Two</B>
  </B>
</A>
"""

#Jordan Gottlieb test case
JORDAN_GOTTLIEB_050120 = """\
<?xml version="1.0" encoding="UTF-8"?>
<box>this box has bubblewrap, in which is a bag and another box
  <bubblewrap>
    <box>stuff</box>
    <bag>bag inside the bubblewrap?</bag>
  </bubblewrap>
</box>"""

QNAME_CLASH_1 = """\
<a xmlns="http://example.com/ns1">
  <a>One</a>
  <a xmlns="http://example.com/ns2">Two</a>
  <b>Three</b>
</a>"""

FDRAKE_NS_EG = """\
<doc xmlns:a="http://xml.python.org/a"
     xmlns:A="http://xml.python.org/a"
     xmlns:b="http://xml.python.org/b"
     a:a="a" b:b="b"
     />
"""

RSALZ_NS_EG = """\
    <S:foo xmlns:S="http://example.com/1">
        <bar>test</bar>
        <bar xmlns='http://example.com/2'>test2</bar>
        <S:bar xmlns:S='http://example.com/2'>test2</S:bar>
    </S:foo>
"""

XSA = """\
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE xsa PUBLIC "-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML"
                     "http://www.garshol.priv.no/download/xsa/xsa.dtd">
<xsa>
  <vendor>
    <name>Fourthought, Inc.</name>
    <email>info@fourthought.com</email>
    <url>http://fourthought.com</url>
  </vendor>
  <product id="FourSuite">
    <name>4Suite</name>
    <version>1.0a1</version>
    <last-release>20030327</last-release>
    <info-url>http://4suite.org</info-url>
    <changes>
 - Begin the 1.0 release cycle
    </changes>
  </product>
</xsa>
"""

XMLDECL = '<?xml version="1.0" encoding="UTF-8"?>\n'

#See http://www-128.ibm.com/developerworks/xml/library/x-namcar.html
BORDERLINE = """<org>
  <a:employee xmlns:a='urn:bogus:ns'>EP</a:employee>
  <b:employee xmlns:b='urn:bogus:ns'>TSE</b:employee>
</org>"""

BORDERLINE_DEFAULT = """<html xmlns="http://www.w3.org/1999/xhtml" xmlns:html="http://www.w3.org/1999/xhtml">
    <p>boo</p>
</html>
"""

NEUROTIC = """
<h:memo xmlns:a='urn:bogus:ns'>
  <h:body xmlns:a='http://www.w3.org/1999/xhtml'>
    Now hear <h:i>this</h:i>
  </h:body>
</h:memo>"""

PSYCHOTIC = """<org xmlns:a='urn:bogus:ns' xmlns:b='urn:bogus:ns'>
  <a:employee>EP</a:employee>
  <b:employee>TSE</b:employee>
</org>
"""

SPECIAL_GIS_1 = """<class class="1">
  <ns/>
  <prefix/>
  <qname/>
  <ename/>
  <namer/>
  <naming_rule/>
</class>
"""

SPECIAL_GIS_2 = """<for for="1">
  <class/>
  <class/>
</for>
"""

from amara.bindery import RESERVED_NAMES
#Names that could potentially cause problems, usually because they could
#interfere with the trickery in bindery.create_element
TRICKY_NAMES = ['instance', 'local', 'qname', 'ns', 'globals', 'ename']


class TestInterestingXmlPatterns(unittest.TestCase):
    #def setUp(self):
    #    return

    def testMixed(self):
        doc = amara.parse(ROOT_MIXED_1)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(list(doc.A.B)), 1)
        self.assertEqual(len(list(doc.A.xml_child_elements)), 1)
        self.assertEqual(list(doc.A.xml_properties.keys()), [u'B'])
        self.assertEqual(len(doc.A.xml_children), 3)
        self.assertEqual(unicode(doc.A), u'xy')
        doc = amara.parse(ROOT_MIXED_2)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(list(doc.A.B)), 1)
        self.assertEqual(len(list(doc.A.xml_child_elements)), 1)
        self.assertEqual(list(doc.A.xml_properties.keys()), [u'B'])
        self.assertEqual(len(doc.A.xml_children), 3)
        self.assertEqual(unicode(doc.A), u'x\n  Two\ny')
        self.assertEqual(doc.A.xml_child_text, u'x\n  \ny')
        return

    def testRootGiInChild(self):
        doc = amara.parse(ROOT_GI_IN_CHILD)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(list(doc.A.A)), 1)
        self.assertEqual(len(list(doc.A.B)), 1)
        self.assertEqual(unicode(doc.A.A), u'One')
        self.assertEqual(unicode(doc[u'A'].A), u'One')
        self.assertEqual(unicode(doc.A[u'A']), u'One')
        self.assertEqual(unicode(doc[u'A'][u'A']), u'One')
        self.assertEqual(unicode(doc.A.B), u'Two')
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testRootGiInChildGrandchild(self):
        doc = amara.parse(ROOT_GI_IN_CHILD_GRANDCHILD)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(list(doc.A.A)), 1)
        self.assertEqual(len(list(doc.A.B)), 1)
        self.assertEqual(len(list(doc.A.A.A)), 1)
        self.assertEqual(len(list(doc.A.A.B)), 1)
        self.assertEqual(len(list(doc.A.B.A)), 1)
        self.assertEqual(len(list(doc.A.B.B)), 1)
        self.assertEqual(unicode(doc.A.A.A), u'A One')
        self.assertEqual(unicode(doc.A.A.B), u'A Two')
        self.assertEqual(unicode(doc.A.B.A), u'B One')
        self.assertEqual(unicode(doc.A.B.B), u'B Two')
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testJordanGottlieb050120(self):
        doc = amara.parse(JORDAN_GOTTLIEB_050120)
        self.assertEqual(len(list(doc.box)), 1)
        self.assertEqual(len(list(doc.box.bubblewrap)), 1)
        self.assertEqual(len(list(doc.box.bubblewrap.box)), 1)
        self.assertEqual(len(list(doc.box.bubblewrap.bag)), 1)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testQnameClash(self):
        doc = amara.parse(QNAME_CLASH_1)
        NS1 = u'http://example.com/ns1'
        NS2 = u'http://example.com/ns2'
        self.assertEqual(len(list(doc.a)), 1)
        self.assertEqual(len(list(doc.a.a)), 1)
        self.assertEqual(len(list(doc.a.a_)), 1)
        self.assertEqual(len(list(doc.a.b)), 1)
        self.assertEqual(unicode(doc.a.a), u'One')
        self.assertEqual(unicode(doc.a.a_), u'Two')
        self.assertEqual(unicode(doc.a.b), u'Three')
        self.assertEqual(unicode(doc[NS1, u'a'].a), u'One')
        self.assertEqual(unicode(doc[NS1, u'a'][NS1, u'a']), u'One')
        self.assertEqual(unicode(doc.a[NS1, u'a']), u'One')
        self.assertEqual(doc.xml(), XMLDECL+QNAME_CLASH_1)
        return

    def testQueryDefaultNs(self):
        doc = amara.parse(BORDERLINE_DEFAULT)
        #print doc.xmlns_prefixes
        self.assertEqual(len(doc.xml_xpath(u"//html:html")), 1)
        return

    def testSpecialGis1(self):
        doc = amara.parse(SPECIAL_GIS_1)
        self.assertEqual(len(list(doc.class_)), 1)
        self.assertEqual(doc.class_.class_, u"1")
        self.assertEqual(len(doc.xml_xpath(u"/class")), 1)
        return

    def testSpecialGis2(self):
        doc = amara.parse(SPECIAL_GIS_2)
        self.assertEqual(len(list(doc.for_)), 1)
        self.assertEqual(len(list(doc.for_.class_)), 2)
        self.assertEqual(len(doc.xml_xpath(u"/for")), 1)
        self.assertEqual(len(doc.xml_xpath(u"/for/class")), 2)
        return

    def testReservedNames(self):
        for name in RESERVED_NAMES:
            doc = amara.parse('<%s/>'%(name))
            self.assertEqual(len(list(getattr(doc, name+'_'))), 1)
            doc = amara.parse('<%(name)s><%(name)s/></%(name)s>'%{'name': name})
            self.assertEqual(len(list(getattr(doc, name+'_'))), 1)
            self.assertEqual(len(list(getattr(getattr(doc, name+'_'), name+'_'))), 1)
        return

    def testTrickyNames(self):
        for name in TRICKY_NAMES:
            doc = amara.parse('<%s/>'%(name))
            self.assertEqual(len(list(getattr(doc, name))), 1)
            doc = amara.parse('<%(name)s><%(name)s/></%(name)s>'%{'name': name})
            self.assertEqual(len(list(getattr(doc, name))), 1)
            self.assertEqual(len(list(getattr(getattr(doc, name), name))), 1)
        return


    def FIXMEtestDoctypeInfo(self):
        doc = amara.parse(XSA)
        self.assertEqual(doc.xml_pubid, u'-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML')
        self.assertEqual(doc.xml_sysid, u'http://www.garshol.priv.no/download/xsa/xsa.dtd')
        self.assertEqual(doc.xml_doctype_name, u'xsa')
        return


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
import os
try:
    # Allow Ft.Lib.DistExt to be used even in setuptools is installed.
    if 'AMARA_FTSETUP' in os.environ:
        raise ImportError
    from setuptools import setup, find_packages
    kw = {
          'install_requires': ['4Suite-XML>=1.0.2', 'python-dateutil'],
           'entry_points': {
               'console_scripts': [
                  'scimitar = amara.scimitar:main',
                  'flextyper = amara.flextyper:main',
                  'trimxml = amara.trimxml:main',
                  ],
               },
          }
except ImportError:
    from Ft.Lib.DistExt import *
    kw = {'download_url': 'ftp://ftp.4suite.org/pub/Amara/',
          'license_file': 'COPYING',
          'scripts': [Script('scimitar', 'amara.scimitar', function='main'),
                      Script('flextyper', 'amara.flextyper', function='main'),
                      Script('trimxml', 'amara.trimxml', function='main'),
                      ],
          'doc_files': [File('ACKNOWLEDGEMENTS'),
                        File('CHANGES'),
                        File('COPYING'),
                        File('TODO'),
                        File('docs/quickref.txt'),
                        Document('docs/manual.xml', 'docbook', title='Users Manual'),
                        ],
          'devel_files': [FileList('tests/bindery', ['test/bindery'], recursive=True),
                          FileList('tests/domtools', ['test/domtools'], recursive=True),
                          FileList('tests/scimitar', ['test/scimitar'], recursive=True),
                          ],
          'config_module': 'amara.__config__',
          'manifest_templates': ['include demo/README',
                                 'recursive-include demo *.py *.xml *.rdf *.rng *.conf',
                                 'recursive-include test *.py',
                                 ],
          'validate_templates': ['prune Amara.egg-info',
                                 'exclude MANIFEST.in',
                                 ],

          }

setup(name='Amara',
      version='1.2.0.1',
      description="Amara XML Toolkit: a collection of Python/XML processing tools to complement 4Suite (http://4Suite.org)",
      long_description="A collection of Python/XML processing tools to complement 4Suite (http://4Suite.org).  The components are: Bindery--a data binding tool (fancy way of saying it's a very Pythonic XML API); Scimitar--an implementation of the ISO Schematron schema language for XML, which converts Schematron files to Python scripts; domtools--A set of tools to augment Python DOMs; saxtools--A set of tools to make SAX easier to use in Python",
      url='http://uche.ogbuji.net/tech/4suite/amara/',
      author='Uche Ogbuji',
      author_email='uche.ogbuji@fourthought.com',
      classifiers=[
         #'Development Status :: 4 - Beta',
         'Development Status :: 5 - Production/Stable',
         #'Development Status :: 3 - Alpha',
         'Intended Audience :: Developers',
         'License :: OSI Approved :: Apache Software License',
         'Programming Language :: Python',
         'Topic :: Software Development :: Libraries :: Python Modules',
         'Topic :: Text Processing :: Markup :: XML',
         ],
      package_dir={'amara': 'lib'},
      packages=['amara'],
      **kw)

